/* ifrh.c */

/* Code to produce instantaneous frequency histogram.
 * Author: Robert Mill <rob.mill.uk@gmail.com>
 * (C)2015 University of Sheffield
 */

/* INCLUDE HEADERS */

#include <math.h>
#include <limits.h>
#include <stdlib.h>
#include <string.h>
#include "ifrh.h"


/* FUNCTION DEFINITIONS */

int IFRH_AllocState( IFRH_State *sp_state, const IFRH_Options *sp_opt )
{
    /* Get the number of channels/bins */
    int n_channels = sp_opt->n_channels;
    int n_bins     = sp_opt->n_bins;

    /* Allocate memory to state */
    sp_state->dp_phi_z = calloc( n_channels, sizeof(double) );
    sp_state->dp_hist  = calloc( n_bins, sizeof(double) );

    /* Failed to allocate any particular memory? */
    if( !sp_state->dp_phi_z || !sp_state->dp_hist )
    {
        /* Free memory that was allocated and return failure */
        IFRH_FreeState( sp_state );
        return IFRH_FAILURE;
    }

    /* Return success */
    return IFRH_SUCCESS;
}


void IFRH_FreeState( IFRH_State *sp_state )
{
    /* Free memory allocated to state */
    free( sp_state->dp_phi_z );
    free( sp_state->dp_hist );

    /* Anull pointers */
    sp_state->dp_phi_z = NULL;
    sp_state->dp_hist  = NULL;
}


void IFRH_InitState( IFRH_State *sp_state, const IFRH_Options *sp_opt )
{
    /* Get number of channels and histogram bins */
    int n_channels = sp_opt->n_channels;
    int n_bins     = sp_opt->n_bins;

    /* Zero all memory */
    memset( sp_state->dp_phi_z, (char)0, n_channels * sizeof(double) );
    memset( sp_state->dp_hist, (char)0, n_bins * sizeof(double) );

    /* Zero samples left until output/measure IFR */
    sp_state->n_out_rem = 0;
    sp_state->n_ifr_rem = 0;
}


int IFRH_CalcOutputSize( 
    int n_in,
    const IFRH_State *sp_state,
    const IFRH_Options *sp_opt )
{
    /* Get the number of output columns required */
    return ( n_in + sp_opt->n_out_wait - 1 ) / sp_opt->n_out_wait;
}


void IFRH_Assign(
    int n_channel,
    double d_fq_Hz,
    double *dp_hist,
    const IFRH_Options *sp_opt )
{
    int i;
    int n_bins = sp_opt->n_bins;

    /* Depending on frequency assignment mode: */
    switch( sp_opt->n_ass_mode )
    {
        case IFRH_ASS_DIRECT:
        {
            /* Compute index of cell to assign frequency to */
            int k = floor( ( d_fq_Hz - sp_opt->d_freq_start ) / 
                sp_opt->d_freq_width + 0.5 );

            /* Bin index in range? */
            if( k >= 0 && k < n_bins )
            {
                /* Add to weighted interval histogram */
                dp_hist[k] += sp_opt->n_ifr_wait / sp_opt->dp_taus_s[k]
                    / sp_opt->n_fs_Hz;
            }

            break;
        }

        case IFRH_ASS_REMAP:
        {
            /* Get pointer to map */
            IFRH_Map *sp_map = sp_opt->sp_maps + n_channel;

            /* Get index of unreassigned frequency */
            int ix = floor(
                ( d_fq_Hz - sp_map->d_fq_start ) / 
                sp_map->d_fq_width + 0.5 );

            /* Is index of raw (unreassigned) frequency within range? */
            if( ix >= 0 && ix < sp_map->n_fq_count )
            {
                /* For each element of the pdf indexed i: */
                for( i = 0; i < sp_map->n_bin_count; i++ )
                {
                    /* Compute bin index to add to */
                    int k = i + sp_map->n_bin_start;
                    int m = i + sp_map->n_bin_count * ix;

                    /* Bin index in range? */
                    if( k >= 0 && k < n_bins )
                    {
                        /* Add to weighted frequency histogram */
                        /* (adds an amount of p.d.f. */
                        dp_hist[k] += sp_opt->n_ifr_wait *
                            sp_map->dp_pdf[m] / 
                            sp_opt->dp_taus_s[k] / sp_opt->n_fs_Hz;
                    }
                }
            }
        }
    }
}


void IFRH_Run(
    const double *dp_in_re,
    const double *dp_in_im,
    double *dp_out,
    int n_in_ts,
    IFRH_State *sp_state,
    const IFRH_Options *sp_opt )
{
    int i, j, k;

    /* Copy options */
    int n_fs_Hz    = sp_opt->n_fs_Hz;
    int n_channels = sp_opt->n_channels;
    int n_bins     = sp_opt->n_bins;
    int n_ifr_wait = sp_opt->n_ifr_wait;
    int n_out_wait = sp_opt->n_out_wait;

    /* Copy pointers to options */
    const double *dp_taus_s = sp_opt->dp_taus_s;

    /* Copy pointer to state data */
    double *dp_phi_z = sp_state->dp_phi_z;
    double *dp_hist  = sp_state->dp_hist;

    /* For each input time step i: */
    for( i = 0; i < n_in_ts; i++ )
    {
        /* Time to measure instantaneous frequency? */
        if( sp_state->n_ifr_rem <= 0 )
        {
            /* For each channel j: */
            for( j = 0; j < n_channels; j++ )
            {
                /* Get current sample value */
                double d_x_re = dp_in_re[j];
                double d_x_im = dp_in_im[j];

                /* Computations:
                 * d_phi0  - phase angle of earlier sample
                 * d_phi1  - phase angle of current sample
                 * d_fq_Hz - instantaneous frequency (Hz)
                 */
                double d_phi0  = dp_phi_z[j];
                double d_phi1  = atan2( d_x_im, d_x_re );
                double d_fq_Hz = d_phi1 - d_phi0;

                /* Convert phase difference to IFR */
                d_fq_Hz *= n_fs_Hz / ( 2 * 3.14159 * n_ifr_wait );

                /* Make assignment to histogram based on frequency */
                IFRH_Assign(
                    j,
                    d_fq_Hz,
                    dp_hist,
                    sp_opt );

                /* Set previous angle in channel to current angle */
                dp_phi_z[j] = d_phi1;
            }

            /* Reset IFR wait timer */
            sp_state->n_ifr_rem = n_ifr_wait;
        }

        /* No samples left to output? */
        if( sp_state->n_out_rem == 0 )
        {
            /* Copy histogram to output */
            memcpy( dp_out, dp_hist, n_bins * sizeof(double) );
            dp_out += n_bins;

            /* Reset waiting time according to options */
            sp_state->n_out_rem = n_out_wait;
        }

        /* Decrement waiting times (samples) */
        sp_state->n_ifr_rem--;
        sp_state->n_out_rem--;

        /* For each histogram bin k: */
        for( k = 0; k < n_bins; k++ )
        {
            /* Perform 1st order linear filter (leak) on histogram */
            dp_hist[k] -= dp_hist[k] / ( dp_taus_s[k] * n_fs_Hz );
        }

        /* Shift to read from next time step in input buffer */
        dp_in_re += n_channels;
        dp_in_im += n_channels;
    }
}

/* end of file */
