/* ifrh.h */

/* Code to produce instantaneous frequency histogram.
 * Author: Robert Mill <rob.mill.uk@gmail.com>
 * (C)2015 University of Sheffield
 */

#ifndef IFRH_H_INCLUDED
#define IFRH_H_INCLUDED


/* MACRO DEFINITIONS */

/* Success and failure constants */
#define IFRH_SUCCESS 0  /* success */
#define IFRH_FAILURE 1  /* failure */

/* Assignment mode options */
#define IFRH_ASS_DIRECT 0  /* Use 1/2i */
#define IFRH_ASS_REMAP  1  /* Reassign IFR using map */


/* STRUCTURE DEFINITIONS */

struct IFRH_Map
{
    int n_bin_start;  /* Index of first bin to contribute to */
    int n_bin_count;  /* Number of bins to contribute to */
    int n_fq_count;   /* Number of frequencies to reassign */

    double d_fq_start;  /* First frequency to reassign */
    double d_fq_width;  /* Frequency width of each cell */

    const double *dp_pdf;  /* Probability density function */
};


/* Structure to store IFRH options */
struct IFRH_Options
{
    int n_fs_Hz;      /* Sample rate (Hz) */
    int n_channels;   /* Number of input channels to process */
    int n_bins;       /* Number of histogram bins */
    int n_ass_mode;   /* Frequency assignment mode (see MACROS) */
    int n_out_wait;   /* Number of samples to wait until output */
    int n_ifr_wait;   /* Number of samples to wait compute IFR over */

    double d_freq_start;  /* Centre frequency of first bin */
    double d_freq_width;  /* Width of frequency bins (Hz) */

    const double *dp_taus_s;  /* Time constants to decay each cell */

    struct IFRH_Map *sp_maps;  /* Maps (or NULL if no maps) */
};


/* Structure to store IFRH state */
struct IFRH_State
{
    int n_out_rem;  /* Number of samples remaining until output */
    int n_ifr_rem;  /* Number of samples remaining until IFR computed */

    double *dp_phi_z; /* Phase of earlier input sample in each channel */
    double *dp_hist;  /* Instantaneous frequency histogram */
};


/* TYPE DEFINITIONS */

typedef struct IFRH_Map IFRH_Map;

typedef struct IFRH_State IFRH_State;

typedef struct IFRH_Options IFRH_Options;


/* FUNCTION PROTOTYPES */

int IFRH_AllocState( IFRH_State *sp_state, const IFRH_Options *sp_opt );

void IFRH_FreeState( IFRH_State *sp_state );

void IFRH_InitState( IFRH_State *sp_state, const IFRH_Options *sp_opt );

int IFRH_CalcOutputSize( 
    int n_in,
    const IFRH_State *sp_state,
    const IFRH_Options *sp_opt );

void IFRH_Run(
    const double *dp_in_re,
    const double *dp_in_im,
    double *dp_out,
    int n_in_ts,
    IFRH_State *sp_state,
    const IFRH_Options *sp_opt );

#endif

/* end of file */
