/* swdft.c */

/* Code to produce sliding-window discrete Fourier transform.
 * Author: Robert Mill <rob.mill.uk@gmail.com>
 * (C)2015 University of Sheffield
 */

/* INCLUDE HEADERS */

#include <math.h>
#include <stdlib.h>
#include <string.h>
#include "swdft.h"


/* INTERNAL CONSTANTS */

const double dp_zero [1] = {0};
const double dp_one  [1] = {1};


/* FUNCTION DEFINITIOSN */

void SWD_InitOptions( SWD_Options *sp_opt )
{
    /* Set default options */
    sp_opt->n_dft_len    = 16;
    sp_opt->n_cell_start = 0;
    sp_opt->n_cell_count = 5;
    sp_opt->d_leak       = 1;
    sp_opt->n_fwin_len   = 1;
    sp_opt->n_fringe     = 0;
    sp_opt->dp_fwin_re   = dp_one;
    sp_opt->dp_fwin_im   = dp_zero;
}


int SWD_AllocState( SWD_State *sp_state, const SWD_Options *sp_opt )
{
    /* Get the number of cells (including fringe) */
    int n_cells = sp_opt->n_cell_count + 2*sp_opt->n_fringe;
    int n_buf   = sp_opt->n_dft_len;

    /* Allocate memory to state */
    sp_state->dp_buffer   = calloc( n_buf, sizeof(double) );
    sp_state->dp_cells_re = calloc( n_cells, sizeof(double) );
    sp_state->dp_cells_im = calloc( n_cells, sizeof(double) );
    sp_state->dp_filt_re  = calloc( n_cells, sizeof(double) );
    sp_state->dp_filt_im  = calloc( n_cells, sizeof(double) );

    /* Failed to allocate any particular memory? */
    if( !sp_state->dp_buffer ||
        !sp_state->dp_cells_re ||
        !sp_state->dp_cells_im ||
        !sp_state->dp_filt_re ||
        !sp_state->dp_filt_im )
    {
        /* Free memory that was allocated and return failure */
        SWD_FreeState( sp_state );
        return SWD_FAILURE;
    }

    /* Return success */
    return SWD_SUCCESS;
}


void SWD_FreeState( SWD_State *sp_state )
{
    /* Free memory allocated to state */
    free( sp_state->dp_buffer );
    free( sp_state->dp_cells_re );
    free( sp_state->dp_cells_im );
    free( sp_state->dp_filt_re );
    free( sp_state->dp_filt_im );

    /* Anull pointers */
    sp_state->dp_buffer   = NULL;
    sp_state->dp_cells_re = NULL;
    sp_state->dp_cells_im = NULL;
    sp_state->dp_filt_re  = NULL;
    sp_state->dp_filt_im  = NULL;
}


void SWD_InitState( SWD_State *sp_state, const SWD_Options *sp_opt )
{
    int j;

    /* Get number of cells (incl. fringe) and DFT length */
    int n_cells = sp_opt->n_cell_count + 2*sp_opt->n_fringe;
    int n_len   = sp_opt->n_dft_len;

    /* Set buffer position to last entry */
    sp_state->n_buf_pos = sp_opt->n_dft_len - 1;

    /* Zero cell memory */
    memset( sp_state->dp_cells_re, (char)0, n_cells * sizeof(double) );
    memset( sp_state->dp_cells_im, (char)0, n_cells * sizeof(double) );

    /* For each cell j: */
    for( j = 0; j < n_cells; j++ )
    {
        int k = j + sp_opt->n_cell_start - sp_opt->n_fringe;

        /* Get filter angle for cell */
        double d_arg = 2*M_PI * k / n_len;

        /* Get real and imaginary part of filter */
        sp_state->dp_filt_re[j] = sp_opt->d_leak * cos( d_arg );
        sp_state->dp_filt_im[j] = sp_opt->d_leak * sin( d_arg );
    }
}


void SWD_Run(
    const double *dp_in,
    double *dp_out_re,
    double *dp_out_im,
    int n_in,
    SWD_State *sp_state,
    const SWD_Options *sp_opt )
{
    int i, j, k;

    /* Get number of cells, buffer length, and freq domain window */
    int n_cells = sp_opt->n_cell_count + 2*sp_opt->n_fringe;
    int n_buf   = sp_opt->n_dft_len;
    int n_fwin  = sp_opt->n_fwin_len;

    /* Get position in buffer and measure half freq domain window size */
    int n_pos  = sp_state->n_buf_pos;
    int n_half = ( n_fwin - 1 ) / 2;

    /* Get pointers to complex window data */
    const double *dp_fwin_re = sp_opt->dp_fwin_re;
    const double *dp_fwin_im = sp_opt->dp_fwin_im;

    /* Get pointers to cell and filter values */
    double *dp_re      = sp_state->dp_cells_re;
    double *dp_im      = sp_state->dp_cells_im;
    double *dp_filt_re = sp_state->dp_filt_re;
    double *dp_filt_im = sp_state->dp_filt_im;

    /* For each input sample i: */
    for( i = 0; i < n_in; i++ )
    {
        /* Get the difference between this sample and the last */
        double d_diff = dp_in[i] - sp_state->dp_buffer[n_pos];

        /* For each cell j: */
        for( j = 0; j < n_cells; j++ )
        {
            /* Get the real part of the cell */
            double d_re_new =
                ( dp_re[j] + d_diff ) * dp_filt_re[j] -
                ( dp_im[j] ) * dp_filt_im[j];

            /* Get the imag part of the cell */
            double d_im_new =
                ( dp_re[j] + d_diff ) * dp_filt_im[j] +
                ( dp_im[j] ) * dp_filt_re[j];

            /* Store the complex value */
            dp_re[j] = d_re_new;
            dp_im[j] = d_im_new;
        }

        /* Decrement/wrap the position in the recent sample buffer */
        sp_state->dp_buffer[n_pos] = dp_in[i];
        n_pos = ( n_pos == 0 ? n_buf : n_pos ) - 1;

        /* For each cell j: */
        for( j = 0; j < sp_opt->n_cell_count; j++ )
        {
            /* Set the accumulators to zero */
            dp_out_re[j] = 0;
            dp_out_im[j] = 0;

            /* For each frequency domain window coefficient k: */
            for( k = 0; k < n_fwin; k++ )
            {
                /* Get frequency/cell difference for convolution */
                int h = j - (k - n_half) + sp_opt->n_fringe;

                /* Convolution in window range? */
                if( h >= 0 && h < n_cells )
                {
                    /* Output windowed DFT cell (real part) */
                    dp_out_re[j] +=
                        dp_fwin_re[k] * dp_re[h] - 
                        dp_fwin_im[k] * dp_im[h];

                    /* Output windowed DFT cell (imag part) */
                    dp_out_im[j] +=
                        dp_fwin_re[k] * dp_im[h] +
                        dp_fwin_im[k] * dp_re[h];
                }
            }
        }

        /* Shift pointers ready to output next data */
        dp_out_re += sp_opt->n_cell_count;
        dp_out_im += sp_opt->n_cell_count;
    }

    /* Store position in buffer */
    sp_state->n_buf_pos = n_pos;
}

/* end of file */
