/* zcpa.c */

/* Code to produce zero crossings with peak amplitudes.
 * Author: Robert Mill <rob.mill.uk@gmail.com>
 * (C)2015 University of Sheffield
 */

/* INCLUDE HEADERS */

#include <math.h>
#include <limits.h>
#include <stdlib.h>
#include <string.h>
#include "zcpa.h"


/* FUNCTION DEFINITIONS */

int ZCPA_AllocState( ZCPA_State *sp_state, const ZCPA_Options *sp_opt )
{
    /* Get the number of channels/bins */
    int n_channels = sp_opt->n_channels;
    int n_bins     = sp_opt->n_bins;

    /* Allocate memory to state */
    sp_state->np_zci      = calloc( n_channels, sizeof(int) );
    sp_state->dp_in_z     = calloc( n_channels, sizeof(double) );
    sp_state->dp_peak     = calloc( n_channels, sizeof(double) );
    sp_state->dp_zci_frac = calloc( n_channels, sizeof(double) );
    sp_state->dp_hist     = calloc( n_bins, sizeof(double) );

    /* Failed to allocate any particular memory? */
    if( !sp_state->np_zci ||
        !sp_state->dp_in_z ||
        !sp_state->dp_peak ||
        !sp_state->dp_zci_frac ||
        !sp_state->dp_hist )
    {
        /* Free memory that was allocated and return failure */
        ZCPA_FreeState( sp_state );
        return ZCPA_FAILURE;
    }

    /* Return success */
    return ZCPA_SUCCESS;
}


void ZCPA_FreeState( ZCPA_State *sp_state )
{
    /* Free memory allocated to state */
    free( sp_state->np_zci );
    free( sp_state->dp_in_z );
    free( sp_state->dp_peak );
    free( sp_state->dp_zci_frac );
    free( sp_state->dp_hist );

    /* Anull pointers */
    sp_state->np_zci      = NULL;
    sp_state->dp_in_z     = NULL;
    sp_state->dp_peak     = NULL;
    sp_state->dp_zci_frac = NULL;
    sp_state->dp_hist     = NULL;
}


void ZCPA_InitState( ZCPA_State *sp_state, const ZCPA_Options *sp_opt )
{
    int j;

    /* Get number of channels and histogram bins */
    int n_channels = sp_opt->n_channels;
    int n_bins     = sp_opt->n_bins;

    /* Zero all memory */
    memset( sp_state->dp_in_z, (char)0, n_channels * sizeof(double) );
    memset( sp_state->dp_peak, (char)0, n_channels * sizeof(double) );
    memset( sp_state->dp_zci_frac, (char)0, n_channels * sizeof(double) );
    memset( sp_state->dp_hist, (char)0, n_bins * sizeof(double) );

    /* For each channel j: */
    for( j = 0; j < n_channels; j++ )
    {
        /* Set infinite number of samples since recent zero crossing */
        sp_state->np_zci[j] = 0;
    }

    /* Zero samples left until output */
    sp_state->n_out_rem = 0;
}


int ZCPA_CalcOutputSize( 
    int n_in,
    const ZCPA_State *sp_state,
    const ZCPA_Options *sp_opt )
{
    /* Get the number of output columns required */
    return ( n_in + sp_opt->n_out_wait - 1 ) / sp_opt->n_out_wait;
}


void ZCPA_Assign(
    int n_channel,
    double d_zci_s,
    double d_peak,
    double *dp_hist,
    const ZCPA_Options *sp_opt )
{
    int    i;
    int    n_bins   = sp_opt->n_bins;
    double d_weight = 0;

    /* Depending on peak mode: */
    switch( sp_opt->n_peak_mode )
    {
        case ZCPA_PEAK_UNIT:

            /* All intervals have unit weight */
            d_weight = 1;
            break;

        case ZCPA_PEAK_SQUARE:

            /* Weight contribution by square of peak */
            d_weight = d_peak*d_peak;
            break;
    }

    /* Depending on interval assignment mode: */
    switch( sp_opt->n_ass_mode )
    {
        case ZCPA_ASS_RECIPROCAL:
        {
            double d_fq_Hz = 0.5 / d_zci_s;

            /* Compute index of cell to assign interval to */
            int k = floor( ( d_fq_Hz - sp_opt->d_freq_start ) / 
                sp_opt->d_freq_width + 0.5 );

            /* Bin index in range? */
            if( k >= 0 && k < n_bins )
            {
                /* Add to weighted interval histogram */
                dp_hist[k] += d_zci_s / sp_opt->dp_taus_s[k] * d_weight;
            }

            break;
        }

        case ZCPA_ASS_REMAP:
        {
            /* Get pointer to map */
            ZCPA_Map *sp_map = sp_opt->sp_maps + n_channel;

            /* Get index of unreassigned interval */
            int ix = floor(
                ( d_zci_s - sp_map->d_zci_start ) / 
                sp_map->d_zci_width + 0.5 );

            /* Is index of raw (unreassigned) interval within range? */
            if( ix >= 0 && ix < sp_map->n_zci_count )
            {
                /* For each element of the pdf indexed i: */
                for( i = 0; i < sp_map->n_bin_count; i++ )
                {
                    /* Compute bin index to add to */
                    int k = i + sp_map->n_bin_start;
                    int m = i + sp_map->n_bin_count * ix;

                    /* Bin index in range? */
                    if( k >= 0 && k < n_bins )
                    {
                        /* Add to weighted interval histogram */
                        /* (adds an amount of p.d.f. */
                        dp_hist[k] += sp_map->dp_pdf[m] * d_zci_s / 
                            sp_opt->dp_taus_s[k] * d_weight;
                    }
                }
            }
        }
    }
}


void ZCPA_Run(
    const double *dp_in,
    double *dp_out,
    int n_in_ts,
    ZCPA_State *sp_state,
    const ZCPA_Options *sp_opt )
{
    int i, j, k;

    /* Copy options */
    int n_fs_Hz     = sp_opt->n_fs_Hz;
    int n_channels  = sp_opt->n_channels;
    int n_bins      = sp_opt->n_bins;

    /* Copy pointers to options */
    const double *dp_taus_s = sp_opt->dp_taus_s;

    /* Copy pointer to state data */
    int    *np_zci      = sp_state->np_zci;
    double *dp_in_z     = sp_state->dp_in_z;
    double *dp_peak     = sp_state->dp_peak;
    double *dp_zci_frac = sp_state->dp_zci_frac;
    double *dp_hist     = sp_state->dp_hist;

    /* For each input time step i: */
    for( i = 0; i < n_in_ts; i++ )
    {
        /* For each channel j: */
        for( j = 0; j < n_channels; j++ )
        {
            /* Get current (1) and previous (0) sample value */
            double d_x0 = dp_in_z[j];
            double d_x1 = dp_in[j];

            /* Increment sample since last ZC was seen in this channel */
            np_zci[j]++;

            /* Current sample largest magnitude since ZC? */
            if( fabs( d_x1 ) > dp_peak[j] )
            {
                /* Update peak value in this channel */
                dp_peak[j] = fabs( d_x1 );
            }

            /* Do the previous and current samples form a sign change? */
            if( ( d_x0 >= 0 && d_x1 < 0 ) || ( d_x0 < 0 && d_x1 >= 0 ) )
            {
                /* Computations:
                 * d_frac  - fractional crossing (samples)
                 * d_zci   - zero crossing interval (fractional samples)
                 * d_zci_s - zero crossing interval (seconds)
                 */
                double d_frac  = d_x0 / ( d_x0 - d_x1 );
                double d_zci   = np_zci[j] + d_frac - dp_zci_frac[j];
                double d_zci_s = d_zci / n_fs_Hz;

                /* Make assignment to histogram based on interval/peak */
                ZCPA_Assign(
                    j,
                    d_zci_s,
                    dp_peak[j],
                    dp_hist,
                    sp_opt );

                /* Now reset counters:
                 * np_zci      - number of samples since last ZC
                 * dp_zci_frac - set last fractional ZC to most recent
                 * dp_peak     - reset highest peak since ZC
                 */
                np_zci[j]      = 0;
                dp_zci_frac[j] = d_frac;
                dp_peak[j]     = fabs( d_x1 );
            }

            /* Set previous sample in channel to current sample */
            dp_in_z[j] = dp_in[j];
        }

        /* No samples left to output? */
        if( sp_state->n_out_rem == 0 )
        {
            /* Copy histogram to output */
            memcpy( dp_out, dp_hist, n_bins * sizeof(double) );
            dp_out += n_bins;

            /* Reset waiting time according to options */
            sp_state->n_out_rem = sp_opt->n_out_wait;
        }

        /* Decrement waiting time (samples) */
        sp_state->n_out_rem--;

        /* For each histogram bin k: */
        for( k = 0; k < n_bins; k++ )
        {
            /* Perform 1st order linear filter (leak) on histogram */
            dp_hist[k] -= dp_hist[k] / ( dp_taus_s[k] * n_fs_Hz );
        }

        /* Shift to read from next time step in input buffer */
        dp_in += n_channels;
    }
}

/* end of file */
