/* zcpa.h */

/* Code to produce zero crossings with peak amplitudes.
 * Author: Robert Mill <rob.mill.uk@gmail.com>
 * (C)2015 University of Sheffield
 */

#ifndef ZCPA_H_INCLUDED
#define ZCPA_H_INCLUDED


/* MACRO DEFINITIONS */

/* Success and failure constants */
#define ZCPA_SUCCESS 0  /* success */
#define ZCPA_FAILURE 1  /* failure */

/* Assignment mode options */
#define ZCPA_ASS_RECIPROCAL 0  /* Use 1/2i */
#define ZCPA_ASS_REMAP      1  /* Reassign interval using map */

/* Peak contribution options */
#define ZCPA_PEAK_UNIT   0  /* Each interval contributes one unit */
#define ZCPA_PEAK_SQUARE 1  /* Each interval contributes square peak */


/* STRUCTURE DEFINITIONS */

struct ZCPA_Map
{
    int n_bin_start;  /* Index of first bin to contribute to */
    int n_bin_count;  /* Number of bins to contribute to */
    int n_zci_count;  /* Number of ZCIs to reassign */

    double d_zci_start;  /* First ZCI to reassign */
    double d_zci_width;  /* Width of each ZCI cell */

    const double *dp_pdf;  /* Probability density function */
};


/* Structure to store ZCPA options */
struct ZCPA_Options
{
    int n_fs_Hz;      /* Sample rate (Hz) */
    int n_channels;   /* Number of input channels to process */
    int n_bins;       /* Number of histogram bins */
    int n_ass_mode;   /* Interval assignment mode (see MACROS) */
    int n_peak_mode;  /* Peak mode (see MACROS) */
    int n_out_wait;   /* Number of samples to wait until output */

    double d_freq_start;  /* Centre frequency of first bin */
    double d_freq_width;  /* Width of frequency bins (Hz) */

    const double *dp_taus_s;  /* Time constants to decay each cell */

    struct ZCPA_Map *sp_maps;  /* Maps (or NULL if no maps) */
};


/* Structure to store ZCPA state */
struct ZCPA_State
{
    int n_out_rem;  /* Number of samples remaining until output */

    int *np_zci;  /* Samples since last zero crossing in each channel */

    double *dp_in_z;      /* Last input sample in each channel */
    double *dp_peak;      /* Largest absolute value since last ZC */
    double *dp_zci_frac;  /* Fractional part of last zero crossing */
    double *dp_hist;      /* Peak/interval histogram */
};


/* TYPE DEFINITIONS */

typedef struct ZCPA_Map ZCPA_Map;

typedef struct ZCPA_State ZCPA_State;

typedef struct ZCPA_Options ZCPA_Options;


/* FUNCTION PROTOTYPES */

int ZCPA_AllocState( ZCPA_State *sp_state, const ZCPA_Options *sp_opt );

void ZCPA_FreeState( ZCPA_State *sp_state );

void ZCPA_InitState( ZCPA_State *sp_state, const ZCPA_Options *sp_opt );

int ZCPA_CalcOutputSize( 
    int n_in,
    const ZCPA_State *sp_state,
    const ZCPA_Options *sp_opt );

void ZCPA_Run(
    const double *dp_in,
    double *dp_out,
    int n_in_ts,
    ZCPA_State *sp_state,
    const ZCPA_Options *sp_opt );

#endif

/* end of file */
