function fpdf = ifr_pdf(Mrho, fqs, dt)
% IFR_PDF Instantaneous frequency probability density function.
%    IFR_PDF(MRHO, FQS, DT) returns the p.d.f. for the instantaneous
%    frequency distribution for a Gaussian random process with
%    autocorrelation MRHO, where MRHO is an additive mixture of modulated
%    Gaussians. The frequencies are given in FQS, in Hertz. DT specifies
%    the sampling interval.
% 
% Author: Robert Mill <rob.mill.uk@gmail.com>
% (C)2015 University of Sheffield
% Part of: (I)nstantaneous (FR)requency
%
% See also IFR_SIN_HIST.


% Convert frequencies to angle differences
phis = fqs * 2*pi*dt;

% Get Hilbert transform of the autocorrelation and return one sample
Mrhoh = mgm_hilbert(Mrho, 'full');
rho1  = mgm_eval(Mrhoh, dt);

% Compute auxilliary values
q1 = abs(rho1) * cos(phis - angle(rho1));
q2 = q1./sqrt(1 - q1.^2);

% Compute frequency pdf
fpdf = (1 - abs(rho1)^2)/2/pi .* (q2.*(atan(q2)+pi/2) + 1)./(1 - q1.^2);
fpdf = fpdf * 2*pi*dt;

% end of file %
