function ifrh = ifr_sin_hist(cfg, N, bins, echo)
% IFR_SIN_HIST Histogram of instantaneous frequencies.
%    IFR_SIN_HIST(CFG, N, BINS) manufactures a signal according to the
%    specification in CFG of length N (samples) and then collects the
%    instantaneous frequencies of the samples into the histogram with bin
%    centres BINS (Hertz).
%
%    The signal is created using IFR_SIN_STATS. For a description of what
%    is required in the CFG structure, see documentation for IFR_SIN_STATS.
%
%    IFR_SIN_HIST(CFG, N, BINS, 'echo') echoes the progress every 1,000,000
%    samples that are processed.
% 
% Author: Robert Mill <rob.mill.uk@gmail.com>
% (C)2015 University of Sheffield
% Part of: (I)nstantaneous (FR)requency
%
% See also IFR_PDF, IFR_SIN_STATS.


% Use defaults?
if nargin < 4 || isempty(echo), echo = 'off'; end

% Reserve space for histogram
ifrh = zeros(size(bins));

% Initial state
st = [];

% While there are more samples to process:
while N > 0

    % Compute size of segment and then sample IFR from random signal
    Nseg      = min(N, 1000000);
    [ifr, st] = ifr_sin_stats(cfg, Nseg, st);

    % Store instantaneous frequencies in histogram
    ifrh = ifrh + hist(ifr, bins);

    % Decrease number of samples left to process
    N = N - Nseg;

    % Echo progress?
    if strcmpi(echo, 'echo')

        % Print progress to console
        fprintf('ifr_sin_hist has %d samples left to process.\n', N);

    end

end

% end of file %
