function [fpdf, etc] = ifr_sin_pdf(Mga, A, sinfHz, fqs, dt)
% IFR_SIN_PDF Instantaneous frequency p.d.f. for sine-in-noise process.
%    IFR_SIN_PDF(MGA, SINA, SINFHZ, IS, DT) returns the p.d.f. of the
%    instantaneous frequency of a sinusoid with amplitude SINA and frequency
%    SINFHZ mixed with a Gaussian random process with autocovariance MGA,
%    where MGA is a modulated Gaussian mixture. The frequencies are given in
%    FQS, in Hertz. DT is the sampling interval across which the IFR
%    measurement is made (seconds).
%
%    [FPDF, ETC] = ZCI_SIN_PDF(...) returns the frequency p.d.f. in FPDF and
%    in ETC returns auxilliary data as a structure with fields
%       .x   - points to evaluate Rayl. components/pulse at
%       .Y   - matrix of Rayleigh components (in columns)
%       .tgt - target pulse
%       .w   - weights applied to Rayleigh components
% 
% Author: Robert Mill <rob.mill.uk@gmail.com>
% (C)2015 University of Sheffield
% Part of: (I)nstantaneous (FR)requency
%
% See also IFR_SIN_HIST.


% Set points to evaluate Rayleigh component at
c = logspace(-2, 0.3, 50);
x = 0:0.001:10;

% Reserve space for Rayleigh components
Y = zeros(numel(x), numel(c));

% For each Rayleigh component n:
for n = 1:length(c)

    % Create Rayleigh component
    Y(:,n) = x/c(n)^2.*exp(-x.^2/c(n)^2/2);

end

% Compute weights for component p.d.f.s using pseudoinverse
tgt = 1/sqrt(2*pi*0.01)*exp(-(x-1).^2/(2*0.01))';
w   = pinv(Y) * tgt;

% Reserve space for frequency p.d.f.
fpdf = zeros(size(fqs));

% Create MGM for pure cosine
Mcos = mgm_cos(A^2, 2*pi*sinfHz);

% For each component n:
for n = 1:numel(c)

    % Generate autocorrelation function
    Mrho = mgm_add(Mga, mgm_scale(Mcos, c(n)^2));
    Mrho = mgm_norm(Mrho, 'zero');

    % Add component p.d.f.
    fpdf = fpdf + w(n)*ifr_pdf(Mrho, fqs, dt);

end

% Return auxilliary data
etc = struct('x', x, 'Y', Y, 'tgt', tgt, 'w', w);

% end of file %
