function [ifr, st] = ifr_sin_stats(cfg, N, st)
% IFR_SIN_STATS Samples of instantaneous frequencies for sine in noise.
%    IFR_SIN_STATS(CFG, N) generates N samples from the sine+noise process
%    described in CFG whose fields are:
%       .Mha    - model (real) impulse response
%       .fsHz   - sample rate (Hz)
%       .sinA   - amplitude of tone (prior to filter)
%       .sinfHz - frequency of tone (Hz)
%    and returns as output is a vector of zero crossing intervals (measured
%    in seconds).
%
%    The noise passed through the filter has a uniform power spectral
%    density with unit power in a 1Hz bandwidth.
%
%    [IFR, ST] = IFR_SIN_STATS(CFG, N, ST) returns the state in ST as a
%    structure containing
%       .z     - filter state
%       .ha    - sampled impulse response
%       .start - number of samples processed
%       .yend  - one samples carried over from previous noise
%    and receives the same state as a parameter.
% 
% Author: Robert Mill <rob.mill.uk@gmail.com>
% (C)2015 University of Sheffield
% Part of: (I)nstantaneous (FR)requency


% Use defaults?
if nargin < 3, st = []; end

% Get complex impulse response
cfg.Mha = mgm_hilbert(cfg.Mha, 'full');

% State not supplied?
if isempty(st)

    % Create an impulse response for the analysis filter
    st.ha    = mgm_impulse(cfg.Mha, cfg.fsHz) / cfg.fsHz;
    st.start = 0;

    % Make sine plus noise
    ts = (st.start:st.start + length(st.ha) - 1)'/cfg.fsHz;
    s  = cfg.sinA*cos(2*pi*cfg.sinfHz*ts);
    n  = sqrt(cfg.fsHz/2) * randn(length(st.ha), 1);

    % Fill filter with sine plus noise
    [~,st.z] = filter(st.ha, 1, s + n);
    st.yend  = [];

end

% Compute length to filter and starting sample
st.start = st.start + N;

% Make sine plus noise
ts = (st.start:st.start + N - 1)'/cfg.fsHz;
s  = cfg.sinA*cos(2*pi*cfg.sinfHz*ts);
n  = sqrt(cfg.fsHz/2) * randn(N, 1);

% Filter and keep overlap with previous signal
[y,st.z] = filter(st.ha, 1, s + n, st.z);
yover    = [st.yend; y];

% Store instantaneous frequencies
ifr = diff(unwrap(angle(yover))) * cfg.fsHz / 2 / pi;

% Keep last sample
st.yend = yover(end);

% end of file %
