function ps = ifrh_mean(cfHz, bwHz, ifopt, MSin)
% IFRH_MEAN Mean instantaneous frequency histogram.
%    IFRH_MEAN(CFHZ, BWHZ, IFOPT, MSIN) returns the mean instantaneous
%    frequency histogram when the noise process described by the mixture
%    model MSIN (see help mgm) is passed into a filter bank with Gaussian
%    shaped pass bands centred on CFHZ and with bandwidths BWHZ. The
%    options for the instantaneous frequency output are given in IFOPT.
% 
% Author: Robert Mill <rob.mill.uk@gmail.com>
% (C)2015 University of Sheffield
% Part of: (I)nstantaneous (FR)requency (H)istogram
%
% See also IFRH_OPT.


% Get edges of bins
nbins  = double(ifopt.n_bins);
bin1Hz = ifopt.d_freq_start;
binwHz = ifopt.d_freq_width;

% Get autocovariance function of filter outputs
Mfb  = mgm_fbank_create(cfHz, bwHz, [], [], 'real');
Mgas = mgm_fbank_autocov(Mfb, MSin);

% Reserve space for pseudospectrum
fqs = bin1Hz + binwHz*(1:nbins);
ps  = zeros(size(fqs));

% For each input channel n:
for n = 1:numel(Mgas)

    % Compute autocovariance and autocorrelation coefficient
    Mrho = mgm_norm(Mgas{n}, 'zero');

    % Generate pdf from single filter output
    fpdf = ifr_pdf(Mrho, fqs, 1/double(ifopt.n_fs_Hz));

    % Add frequency histogram to pseudospectrum (constant weighting)
    ps = ps + fpdf*binwHz;

end

% end of file %
