% MAKE_FIGURE1 Make Figure 1 for JASA paper.
%
% Author: Robert Mill <rob.mill.uk@gmail.com>
% (C)2015 University of Sheffield
% Part of: (MAKE) materials for JASA paper.


% Set figure parameters
scale      = 2;
areacolour = [0.8 0.8 0.8];
linecolour = [0 0 0];

% Set signal characteristics
fsHz = 32768;
cfHz = 1000;
bwHz = 100;
nsec = 300;

% Construct filter model in time and frequency domains
sig = mgm_bw2sig(bwHz, 3, 'time') / sqrt(2);
Mha = mgm_create(1, 1, sig, 0, 2*pi*cfHz, 0, 'real');
MHa = mgm_simplify(mgm_abs2(mgm_fourier(Mha)));

% Scale analysis filter magnitude to get noise p.s.d.
MS = mgm_scale(MHa, 1/2);

% Construct autocovariance and autocorrelation coefficient models
Mga  = mgm_ifourier(MHa);
Mrho = mgm_norm(Mga, 'zero');

% Configure simulation
cfg = struct( ...
    'Mha', Mha, ...
    'fsHz', fsHz, ...
    'sinA', 0, ...
    'sinfHz', 0);

% Generate sampled versions of impulse response, autocorrelation and
% power spectrum
tau = 0:1/fsHz:20e-3;
ha  = mgm_impulse(Mha, fsHz, [], [], 'real');
rho = mgm_eval(Mrho, tau, 'real');
f   = 0:2*cfHz;
S   = mgm_eval(MS, 2*pi*f, 'real');

% Set intervals to evaluate p.d.f. at
i0 = 1/(cfHz * 2^(3/2));
i1 = 1/(cfHz * 2^(1/2));
di = 1e-6;
is = 0:di:1e-3;

% Compute analytical cdf and pdf
aicdf = zci_cdf(Mrho, is);
aipdf = zci_pdf(Mrho, is);

% Apply limits to cdf
aicdf(is >= i1) = 1;
aicdf(is <= i0) = 0;

% Apply limits to pdf
aipdf(is <= i0) = 0;
aipdf(is >= i1) = 0;

% Make empirical interval pdf and cdf
eipdf = zci_sin_hist(cfg, nsec*fsHz, is, 'echo');
nint  = sum(eipdf);
eicdf = cumsum(eipdf) / nint;

% Normalise pdf
eipdf = eipdf / sum(eipdf);
eipdf = eipdf / di;

% Print number of intervals counted
fprintf('%d intervals counted over %d seconds.\n', nint, nsec);

% Set frequency range over which to evaluate IFR c/pdf
dfq = 1;
fqs = 0:dfq:ceil(cfHz * 2);

% Compute analytical IFR pdf
afpdf = ifr_pdf(Mrho, fqs, 1/fsHz);

% Collect empirical IFR pdf
efpdf = ifr_sin_hist(cfg, nsec*fsHz, fqs, 'echo');
nifr  = sum(efpdf);
efpdf = efpdf / sum(efpdf);
efpdf = efpdf / dfq;

% Print number of intervals counted
fprintf('%d IFR samples counted over %d seconds.\n', nifr, nsec);

% Create a figure
figpos = [1 1 scale*[3.3 4.0]];
hFig   = figure(...
    'Units', 'inches', ...
    'Position', figpos, ...
    'Color', 'w', ...
    'InvertHardCopy', 'off', ...
    'PaperPositionMode', 'manual');

% Place axes on figure
axespos = scale*[0.35 3.0 1.25 0.7];
axes(...
    'Units', 'inches', ...
    'Position', axespos, ...
    'LineWidth', scale, ...
    'FontSize', 8*scale, ...
    'Layer', 'top', ...
    'XLim', [0 max(f)/1000], ...
    'YLim', [0 1.1], ...
    'YTick', [0 1]);
hold on;

% Plot power spectrum
plot(f / 1000, S / max(S), 'k', 'Color', linecolour, 'LineWidth', scale);
plot(cfHz*2^(-1/2)*[1 1]/1000, [0 1.1], 'k:', 'LineWidth', scale);
plot(cfHz*2^(+1/2)*[1 1]/1000, [0 1.1], 'k:', 'LineWidth', scale);

% Add labels
% (position y label close to axes)
xlabel('Frequency f \times 10^{-3} (kHz)');
ylabel('|H_a(\omega = 2\pi f)|^2', ...
    'Units', 'normalized', ...
    'Position', [-0.1 0.5 0]);
title('Filter Response')

% Place axes on figure
axespos = scale*[1.95 3.0 1.25 0.7];
axes(...
    'Units', 'inches', ...
    'Position', axespos, ...
    'LineWidth', scale, ...
    'FontSize', 8*scale, ...
    'Layer', 'top', ...
    'XLim', [0 20], ...
    'YLim', [-1.1 1.1], ...
    'YTick', [-1 0 1]);
hold on;

% Plot autocorrelation
plot(tau * 1000, rho, 'k', 'Color', linecolour, 'LineWidth', scale);

% Add labels
xlabel 'Time \tau \times 10^3 (ms)'
ylabel '\rho(\tau)'
title 'Autocorrelation Coefficient'

% Place axes on figure (Interval CDF)
axespos = scale*[0.35 1.7 1.25 0.7];
axes(...
    'Units', 'inches', ...
    'Position', axespos, ...
    'LineWidth', scale, ...
    'FontSize', 8*scale, ...
    'Layer', 'top', ...
    'XLim', [0 1], ...
    'YLim', [0 1.1], ...
    'YTick', [0 1]);
hold on;

% Plot empirical cdf
area(is * 1000, eicdf, ...
    'FaceColor', areacolour, ...
    'EdgeColor', areacolour);

% Plot analytical cdf
plot(is * 1000, aicdf, 'k', 'Color', linecolour, 'LineWidth', scale);
plot(i0*[1 1]*1000, [0 1.1], 'k:', 'LineWidth', scale);
plot(i1*[1 1]*1000, [0 1.1], 'k:', 'LineWidth', scale);

% Add labels
xlabel 'Interval i \times 10^3 (ms)'
ylabel 'Pr(I \leq i)'
title 'ZC Interval c.d.f.'

% Place axes on figure (Interval PDF)
axespos = scale*[1.95 1.7 1.25 0.7];
axes(...
    'Units', 'inches', ...
    'Position', axespos, ...
    'LineWidth', scale, ...
    'FontSize', 8*scale, ...
    'Layer', 'top', ...
    'XLim', [0 1], ...
    'YLim', [0 1.1], ...
    'YTick', [0 1], ...
    'YTickLabel', {'0', 'max'});
hold on;

% Plot empirical pdf
area(is * 1000, eipdf/max(aipdf), ...
    'FaceColor', areacolour, ...
    'EdgeColor', areacolour);

% Plot analytical pdf
plot(is * 1000, aipdf/max(aipdf), 'k', ...
    'Color', linecolour, ...
    'LineWidth', scale);
plot(i0*[1 1]*1000, [0 1.1], 'k:', 'LineWidth', scale);
plot(i1*[1 1]*1000, [0 1.1], 'k:', 'LineWidth', scale);

% Add labels
% (position y label close to axes)
xlabel('Interval i \times 10^3 (ms)');
ylabel('p_I(i)', 'Units', 'normalized', 'Position', [-0.1 0.5 0]);
title('ZC Interval p.d.f.');

% Place axes on figure (IFR PDF)
axespos = scale*[1.95 0.4 1.25 0.7];
axes(...
    'Units', 'inches', ...
    'Position', axespos, ...
    'LineWidth', scale, ...
    'FontSize', 8*scale, ...
    'Layer', 'top', ...
    'XLim', [0 max(fqs)/1000], ...
    'YLim', [0 1.1], ...
    'YTick', [0 1], ...
    'YTickLabel', {'0', 'max'});
hold on;

% Plot empirical pdf
area(fqs / 1000, efpdf/max(efpdf), ...
    'FaceColor', areacolour, ...
    'EdgeColor', areacolour);

% Plot analytical cdf
plot(fqs / 1000, afpdf/max(afpdf), 'k', ...
    'Color', linecolour, ...
    'LineWidth', scale);
plot(cfHz*2^(-1/2)*[1 1]/1000, [0 1.1], 'k:', 'LineWidth', scale);
plot(cfHz*2^(+1/2)*[1 1]/1000, [0 1.1], 'k:', 'LineWidth', scale);

% Add labels
% (position y label close to axes)
xlabel('IFR f \times 10^{-3} (kHz)');
ylabel('p_F(f)', 'Units', 'normalized', 'Position', [-0.1 0.5 0]);
title('Instant. Freq. p.d.f.');

% Add annotation A
annotation( ...
    'textbox', ...
    'Units', 'inches', ...
    'Position', scale*[0.0 3.8 0.2 0.2], ...
    'String', 'A', ...
    'FontSize', 12*scale, ...
    'FontWeight', 'bold', ...
    'LineStyle', 'none');

% Add annotation B
annotation( ...
    'textbox', ...
    'Units', 'inches', ...
    'Position', scale*[1.6 3.8 0.2 0.2], ...
    'String', 'B', ...
    'FontSize', 12*scale, ...
    'FontWeight', 'bold', ...
    'LineStyle', 'none');

% Add annotation C
annotation( ...
    'textbox', ...
    'Units', 'inches', ...
    'Position', scale*[0.0 2.4 0.2 0.2], ...
    'String', 'C', ...
    'FontSize', 12*scale, ...
    'FontWeight', 'bold', ...
    'LineStyle', 'none');

% Add annotation D
annotation( ...
    'textbox', ...
    'Units', 'inches', ...
    'Position', scale*[1.6 2.4 0.2 0.2], ...
    'String', 'D', ...
    'FontSize', 12*scale, ...
    'FontWeight', 'bold', ...
    'LineStyle', 'none');

% Add annotation E
annotation( ...
    'textbox', ...
    'Units', 'inches', ...
    'Position', scale*[1.6 1.2 0.2 0.2], ...
    'String', 'E', ...
    'FontSize', 12*scale, ...
    'FontWeight', 'bold', ...
    'LineStyle', 'none');

% end of file %
