% MAKE_FIGURE2B Make Figure 2 Panel B for JASA paper.
%
% Author: Robert Mill <rob.mill.uk@gmail.com>
% (C)2015 University of Sheffield
% Part of: (MAKE) materials for JASA paper.


% Set figure parameters
scale      = 2;
areacolour = [0.8 0.8 0.8];
linecolour = [0 0 0];

% Set signal characteristics
fsHz = 32768;
cfHz = 1000;
bwHz = 100;
nsec = 300;

% Construct filter model in time and frequency domains
sig = mgm_bw2sig(bwHz, 3, 'time') / sqrt(2);
Mha = mgm_create(1, 1, sig, 0, 2*pi*cfHz, 0, 'real');
MHa = mgm_simplify(mgm_abs2(mgm_fourier(Mha)));

% Scale analysis filter magnitude to get noise p.s.d.
MS = mgm_scale(MHa, 1/2);

% Construct autocovariance and autocorrelation coefficient models
Mga  = mgm_ifourier(MS);
Mrho = mgm_norm(Mga, 'zero');

% Configure simulation
cfg = struct( ...
    'Mha', Mha, ...
    'fsHz', fsHz, ...
    'sinA', 0, ...
    'sinfHz', 0);

% Generate sampled versions of impulse response, autocorrelation and
% power spectrum
tau = 0:1/fsHz:20e-3;
ha  = mgm_impulse(Mha, fsHz, [], [], 'real');
rho = mgm_eval(Mrho, tau, 'real');
f   = 0:2*cfHz;
Ha  = mgm_eval(MHa, 2*pi*f, 'real');

% Set interval range over which to evaluate interval c/pdf
i0 = 1/(cfHz * 2^(3/2));
i1 = 1/(cfHz * 2^(1/2));
is = linspace(i0, i1, 200);
di = is(2) - is(1);

% Set squared amplitude range
as = linspace(0, 0.04, 800);
da = as(2) - as(1);

% Collect empirical interval pdf and cdf
eiapdf = zci_sin_pk_hist(cfg, nsec*fsHz, is, as, 'echo');
nint   = sum(eiapdf(:));

% Normalise pdf
eiapdf = eiapdf / sum(eiapdf(:));
eiapdf = eiapdf / (di * da);

% Knock out edges of histogram
eiapdf([1 end], :) = 0;

% Print number of intervals counted
fprintf('%d intervals counted over %d seconds.\n', nint, nsec);

% Component analytical PDF
aiapdf = zci_pk_pdf(Mga, is, as, 'grid');

% Create a figure
figpos = [1 1 scale*[3.3 2.85]];
hFig   = figure(...
    'Units', 'inches', ...
    'Position', figpos, ...
    'Color', 'w', ...
    'InvertHardCopy', 'off', ...
    'PaperPositionMode', 'manual');

% Place axes on figure (Autocorrelation)
axespos = scale*[0.4 1.15 1.75 1.3];
axes(...
    'Units', 'inches', ...
    'Position', axespos, ...
    'LineWidth', scale, ...
    'FontSize', 8*scale, ...
    'Layer', 'top');
hold on; axis xy; 

% Create contours (empirical) and make broad, gray
cs = 0.1:0.2:0.9;
hs = util_contours(is * 1000, as, eiapdf'/max(max(aiapdf)), cs, 10);
set(hs, 'Color', [0.8 0.8 0.8], 'LineWidth', 3.0*scale);

% Create contours (analytical) and make thin, black
hs = util_contours(is * 1000, as, aiapdf'/max(max(aiapdf)), cs, 1);
set(hs, 'Color', 'k', 'LineWidth', scale);

% Plot vertical line at centre frequency (dotted)
plot(500/cfHz*[1 1], [0 max(as)], 'k:', 'LineWidth', scale);

% Set axes limits and ticks
set(gca, ...
    'XLim', [i0 i1] * 1000, ...
    'YLim', [0 0.01], ...
    'YTick', [0 0.01]);

% Add labels
% (position y label close to axes)
ylabel('Squared peak a', 'Units', 'normalized', 'Position', [-0.1 0.5 0]);
title('Interval-Peak p.d.f., p_{IA}(i, a)');

% Place axes on figure (Autocorrelation)
axespos = scale*[2.4 1.15 0.75 1.3];
axes(...
    'Units', 'inches', ...
    'Position', axespos, ...
    'LineWidth', scale, ...
    'FontSize', 8*scale, ...
    'Layer', 'top');
hold on; box off

% Plot: area (intervals: empirical)
%       solid curve (intervals: analytical)
area(sum(eiapdf*di, 1), as, ...
    'FaceColor', areacolour, ...
    'EdgeColor', areacolour, ...
    'LineWidth', scale);
plot(sum(aiapdf*di, 1), as, ...
    'Color', linecolour, ...
    'LineWidth', scale);

% Set X/Y limits and ticks
set(gca, ...
    'XLim', [0 400], 'XTick', [0 400], ...
    'YTick', [0 0.01], 'YLim', [0 0.01]);

% Add x label
xlabel 'p_A(a)'

% Place axes on figure (Autocorrelation)
axespos = scale*[0.4 0.4 1.75 0.5];
axes(...
    'Units', 'inches', ...
    'Position', axespos, ...
    'LineWidth', scale, ...
    'FontSize', 8*scale, ...
    'Layer', 'top');
hold on; box off

% Plot: area (peaks: empirical)
%       solid curve (peaks: analytical)
%       vertical line at centre frequency (dotted)
area(is * 1000, sum(eiapdf*da, 2), ...
    'FaceColor', areacolour, ...
    'EdgeColor', areacolour, ...
    'LineWidth', scale);
plot(is * 1000, sum(aiapdf*da, 2), ...
    'Color', linecolour, ...
    'LineWidth', scale);
plot(500/cfHz*[1 1], [0 50000], 'k:', 'LineWidth', scale);

% Set axis limits
set(gca, ...
    'XLim', [i0 i1] * 1000, ...
    'YLim', [0 5*10^4], ...
    'YTick', [0 50000], 'YTickLabel', {0 '50000'});

% Add labels to axes
xlabel('Interval i \times 10^3 (ms)');
ylabel('p_I(i)', 'Units', 'normalized', 'Position', [-0.1 0.5 0]);

% Add annotation B
annotation( ...
    'textbox', ...
    'Units', 'inches', ...
    'Position', scale*[0.1 2.6 0.2 0.2], ...
    'String', 'B', ...
    'FontSize', 12*scale, ...
    'FontWeight', 'bold', ...
    'Margin', 0, ...
    'LineStyle', 'none');

% end of file %
