% MAKE_FIGURE3 Make Figure 3 for JASA paper.
%
% Author: Robert Mill <rob.mill.uk@gmail.com>
% (C)2015 University of Sheffield
% Part of: (MAKE) materials for JASA paper.


% Set figure parameters
scale      = 2;
areacolour = [0.8 0.8 0.8];
linecolour = [0 0 0];

% Set noise characteristics
fsHz = 32768;
cfHz = 1000;
bwHz = 100;
nsec = 300;

% Set sine characteristics
sinA   = sqrt(1/2)*90;
sinfHz = 950;

% Construct filter model in time and frequency domains
sig = mgm_bw2sig(bwHz, 3, 'time') / sqrt(2);
Mha = mgm_create(1, 1, sig, 0, 2*pi*cfHz, 0, 'real');
MHa = mgm_simplify(mgm_abs2(mgm_fourier(Mha)));

% Construct autocovariance and autocorrelation coefficient models
attn  = mgm_eval(MHa, 2*pi*sinfHz, 'real');
Mga   = mgm_scale(mgm_ifourier(MHa), 1/2);
Mrho  = mgm_norm(Mga, 'zero');

% Add cosine directly to make (naive) signal+noise model
Mgan  = mgm_add(Mga, mgm_cos(sinA^2/2*attn, 2*pi*sinfHz));
Mrhon = mgm_norm(Mgan, 'zero');

% Generate sampled versions of impulse response, autocorrelation and
tau = 0:1/fsHz:20e-3;
ha  = mgm_impulse(Mha, fsHz);
rho = mgm_eval(Mrho, tau, 'real');
f   = 0:2*cfHz;
Ha  = mgm_eval(MHa, 2*pi*f, 'real');

% Set interval range over which to evaluate interval c/pdf
i0 = 1/(cfHz * 2^(3/2));
i1 = 1/(cfHz * 2^(1/2));
di = 0.5e-6;
is = i0:di:i1;

% Compute naive and Rayleigh analytical interval pdf and cdf
nipdf        = zci_pdf(Mrhon, is);
[ripdf, etc] = zci_sin_pdf(Mga, sqrt(attn)*sinA, sinfHz, is);

% Configure signal-and-noise model
cfg.Mha    = Mha;
cfg.fsHz   = fsHz;
cfg.sinA   = sinA;
cfg.sinfHz = sinfHz;

% Collect empirical interval pdf and cdf
eipdf = zci_sin_hist(cfg, nsec*fsHz, is, 'echo');
nint  = sum(eipdf);

% Normalise pdf
eipdf = eipdf / sum(eipdf);
eipdf = eipdf / di;

% Print number of intervals counted
fprintf('%d intervals counted over %d seconds.\n', nint, nsec);

% Create a figure
figpos = [1 1 scale*[3.3 2.0]];
figure(...
    'Units', 'inches', ...
    'Position', figpos, ...
    'Color', 'w', ...
    'InvertHardCopy', 'off', ...
    'PaperPositionMode', 'manual');

% Place axes on figure (Interval PDF)
axespos = scale*[0.4 0.4 1.6 1.3];
axes(...
    'Units', 'inches', ...
    'Position', axespos, ...
    'LineWidth', scale, ...
    'FontSize', 8*scale, ...
    'Layer', 'top', ...
    'XLim', [475 565], ...
    'YLim', [0 1.1], ...
    'YTick', [0 1], ...
    'YTickLabel', {'0', 'max'});
hold on;

% Plot empirical pdf
area(is * 1e6, eipdf/max(ripdf), ...
    'FaceColor', areacolour, ...
    'EdgeColor', areacolour);

% Plot analytical pdf (naive)
plot(is * 1e6, nipdf/max(ripdf), 'k', ...
    'Color', 1.5*linecolour, ...
    'LineWidth', scale);

% Plot analytical pdf (Rayleigh)
plot(is * 1e6, ripdf/max(ripdf), 'k:', ...
    'Color', linecolour, ...
    'LineWidth', scale);

% Add labels
% (position y label close to axes)
xlabel('Interval i \times 10^6 (\mu s)');
ylabel('p_I(i)', 'Units', 'normalized', 'Position', [-0.1 0.5 0]);
title('ZC Interval p.d.f.');

% Place axes on figure (summed Rayleigh components)
axespos = scale*[2.3 0.4 0.9 0.4];
axes(...
    'Units', 'inches', ...
    'Position', axespos, ...
    'LineWidth', scale, ...
    'FontSize', 8*scale, ...
    'Layer', 'top', ...
    'XLim', [0 2], ...
    'YLim', [0 4], 'YTick', [0 4]);
hold on;

% Plot summed Rayleigh components
plot(etc.x, etc.tgt, 'k', 'Color', [0.8 0.8 0.8], 'LineWidth', scale);
plot(etc.x, etc.Y*etc.w, 'k', 'LineWidth', scale);

% Place axes on figure (Rayleigh components)
axespos = scale*[2.3 1.0 0.9 0.7];
axes(...
    'Units', 'inches', ...
    'Position', axespos, ...
    'LineWidth', scale, ...
    'FontSize', 8*scale, ...
    'Layer', 'top', ...
    'XLim', [0 2], 'XTick', []);
hold on;

% Plot weightsed Rayleigh components
plot(etc.x, etc.Y*diag(etc.w), 'k', 'LineWidth', 0.5 * scale);

% Add annotation A
annotation( ...
    'textbox', ...
    'Units', 'inches', ...
    'Position', scale*[0.0 1.8 0.2 0.2], ...
    'String', 'A', ...
    'FontSize', 12*scale, ...
    'FontWeight', 'bold', ...
    'LineStyle', 'none');

% Add annotation B
annotation( ...
    'textbox', ...
    'Units', 'inches', ...
    'Position', scale*[1.9 1.8 0.2 0.2], ...
    'String', 'B', ...
    'FontSize', 12*scale, ...
    'FontWeight', 'bold', ...
    'LineStyle', 'none');

% end of file %
