% MAKE_FIGURE6 Make Figure 6 for JASA paper.
%
% Author: Robert Mill <rob.mill.uk@gmail.com>
% (C)2015 University of Sheffield
% Part of: (MAKE) materials for JASA paper.


% Set gap between panel bases
ygap = 0.7;

% Set figure parameters
scale      = 2;
areacolour = [0.8 0.8 0.8];
linecolour = [0 0 0];

% Set analysis bandwidth
bwHz = 64;

% Set sliding-window Fourier transform parameters
fsHz  = 32768;
ndft  = 1024;
nsec  = 100;
cell1 = 32;
celln = 33;

% Set histogram bins parameters
nbins  = 256;
binwHz = 4;
bin1Hz = 1024;

% Create a filter to shape noise
Mhs = mgm_create(1, 20, 1/50, 0, 2*pi*1500, 0, 'real');
Mhs = mgm_add(Mhs, mgm_create(1, 50, 1/50, 0, 2*pi*1250, 0, 'real'));
Mhs = mgm_add(Mhs, mgm_create(1, 3000, 1/3000, 0, 2*pi*1000, 0, 'real'));
Mhs = mgm_add(Mhs, mgm_create(1, 1000, 1/1000, 0, 2*pi*1700, 0, 'real'));

% Get power spectrum noise
MSs = mgm_abs2(mgm_fourier(Mhs));
MSs = mgm_simplify(MSs);

% Set frequencies to plot spectrum at
fqHz = 750:2250;
Ss   = mgm_eval(MSs, 2*pi*fqHz, 'real');
SsdB = 10*log10(Ss);

% Sample impulse response
hs = mgm_impulse(Mhs, fsHz);

% Get SWDFT options structure
swopt = swdft_opt;

% Set basic parameters
swopt.n_fs_Hz      = int32(fsHz);
swopt.n_dft_len    = int32(ndft);
swopt.n_cell_start = int32(cell1);
swopt.n_cell_count = int32(celln);
swopt.d_bw_Hz      = bwHz;

% Apply window to SWDFT and add fringe
swopt = swdft_window(swopt, 'gauss');
swopt = swdft_fringe(swopt);

% Get centre frequencies of analysis filters
cfHz = (cell1+(0:celln-1)) * fsHz/ndft;

% Set options of ZCIH
zcopt              = zcpa_opt;
zcopt.n_channels   = int32(celln);
zcopt.n_bins       = int32(nbins);
zcopt.n_ass_mode   = int32(0);
zcopt.n_peak_mode  = int32(0);
zcopt.d_freq_start = bin1Hz;
zcopt.d_freq_width = binwHz;
zcopt.dp_taus_s    = ones(256, 1);

% Set options of ZCPA
zpopt             = zcopt;
zpopt.n_peak_mode = int32(1);

% Set options of IFRH
ifopt              = ifrh_opt;
ifopt.n_channels   = int32(celln);
ifopt.n_bins       = int32(nbins);
ifopt.n_ass_mode   = int32(0);
ifopt.d_freq_start = bin1Hz;
ifopt.d_freq_width = binwHz;
ifopt.dp_taus_s    = ones(256, 1);

% Get bin centres (Hz)
binsHz = (0:nbins-1) * binwHz + bin1Hz;

% Turn filter to column and get its length
hs   = hs(:)/fsHz;
hlen = length(hs);

% N samples left to process
N = nsec * fsHz;

% Fill filter with noise
n     = sqrt(fsHz) * randn(fsHz, 1);
[y,z] = filter(hs, 1, n);

% Run SWDFT->ZCIH/ZCPA
[DFT, swst] = swdft_run(y, swopt);
[~, zcst]   = zcpa_run(DFT/fsHz, zcopt);
[~, zpst]   = zcpa_run(DFT/fsHz, zpopt);
[~, ifst]   = ifrh_run(DFT/fsHz, ifopt);

% Set accumulators to sum spectrum representation over time
DFTm  = 0;
ZCIHm = 0;
ZCPAm = 0;
IFRHm = 0;

% While there are more samples to process:
while N > 0

    % Filter and keep overlap with previous signal
    n     = sqrt(fsHz) * randn(min(100000, N), 1);
    [y,z] = filter(hs, 1, n, z);

    % Run SWDFT
    [DFT, swst]  = swdft_run(y, swopt, swst);
    DFT          = DFT / fsHz;

    % Run ZCIH/ZCPA/IFRH
    [ZCIH, zcst] = zcpa_run(DFT, zcopt, zcst);
    [ZCPA, zpst] = zcpa_run(DFT, zpopt, zpst);
    [IFRH, ifst] = ifrh_run(DFT, ifopt, ifst);

    % Add pseudospectra to running totals
    DFTm  = DFTm + sum(abs(DFT).^2, 2);
    ZCIHm = ZCIHm + sum(ZCIH, 2);
    ZCPAm = ZCPAm + sum(ZCPA, 2);
    IFRHm = IFRHm + sum(IFRH, 2);

    % Free memory explicitly
    clear DFT ZCIH ZCPA IFRH

    % Decrease number of samples left to process
    N = N - length(n);

    % Print progress to console
    fprintf('Script has %d samples left to process.\n', N);

end

% Set accumulators to sum spectrum representation over time
DFTm  = DFTm / (nsec*fsHz);
ZCIHm = ZCIHm / (nsec*fsHz);
ZCPAm = ZCPAm / (nsec*fsHz);
IFRHm = IFRHm / (nsec*fsHz);

% Create a model for the filterbank and the autocovariance function at the
% output of each channel (cell arrays)
Mfb = swdft_mgm(swopt);
Mga = mgm_fbank_autocov(Mfb, MSs);

% Analytically determine the average DFT magnitude-square, ZCIH and ZCPA
aDFTm  = cellfun(@(M) mgm_eval(M, 0, 'real'), Mga);
aZCIHm = zcpa_mean(cfHz, bwHz, zcopt, MSs);
aZCPAm = zcpa_mean(cfHz, bwHz, zpopt, MSs);
aIFRHm = ifrh_mean(cfHz, bwHz, zpopt, MSs);

% Create figure
figpos = [1 1 scale*[3.2 4.3]];
figure(...
    'Units', 'inches', ...
    'Position', figpos, ...
    'Color', 'w', ...
    'InvertHardCopy', 'off', ...
    'PaperPositionMode', 'manual');

% Create axes for power spectrum
axespos = scale*[0.55 0.7+4*ygap 1.8 0.6];
axes(...
    'Units', 'inches', ...
    'Position', axespos, ...
    'LineWidth', scale, ...
    'FontSize', 8*scale, ...
    'XLim', [750 2250], 'XTickLabel', [], ...
    'YLim', [-6 8], ...
    'TickDir', 'out', ...
    'Layer', 'top');
hold on;

% Plot power spectrum
plot(fqHz, SsdB, 'k', 'LineWidth', scale);

% Create axes for plotting analysis bands
axespos = scale*[0.55 0.4+4*ygap 1.8 0.2];
axes(...
    'Units', 'inches', ...
    'Position', axespos, ...
    'LineWidth', scale, ...
    'FontSize', 8*scale, ...
    'XLim', [750 2250], ...
    'YLim', [0 1.1], ...
    'YTick', [], ...
    'TickDir', 'out', ...
    'Layer', 'top');
hold on;

% For each analysis filter n:
for n = 1:length(Mfb)

    % Evaluate analysis filter and plot in gray
    MHs = mgm_abs2(mgm_fourier(Mfb{n}));
    Hs  = mgm_eval(MHs, 2*pi*fqHz, 'real');
    plot(fqHz, Hs/max(Hs), ...
        'LineWidth', scale, ...
        'Color', [0.8 0.8 0.8]);

end

% Add black filter example
MHs = mgm_abs2(mgm_fourier(Mfb{1}));
Hs  = mgm_eval(MHs, 2*pi*fqHz, 'real');
plot(fqHz, Hs/max(Hs), 'k', 'LineWidth', scale);

% Add annotation A
annotation( ...
    'textbox', ...
    'Units', 'inches', ...
    'Position', scale*[0 1.3+4*ygap 0.2 0.2], ...
    'String', 'A', ...
    'FontSize', 12*scale, ...
    'FontWeight', 'bold', ...
    'LineStyle', 'none');

% Add text description
annotation( ...
    'textbox', ...
    'Units', 'inches', ...
    'Position', scale*[2.4 0.4+4*ygap 0.5 0.2], ...
    'String', 'Filters', ...
    'FontSize', 8*scale, ...
    'VerticalAlignment', 'middle', ...
    'LineStyle', 'none');

% Add text description
annotation( ...
    'textbox', ...
    'Units', 'inches', ...
    'Position', scale*[2.4 0.8+4*ygap 0.8 0.4], ...
    'String', 'Input Power Spectral Density', ...
    'FontSize', 8*scale, ...
    'VerticalAlignment', 'middle', ...
    'LineStyle', 'none');

% Create axes for plotting mean power spectra (low-resolution)
axespos = scale*[0.55 0.4+3*ygap 1.8 0.4];
axes(...
    'Units', 'inches', ...
    'Position', axespos, ...
    'LineWidth', scale, ...
    'FontSize', 8*scale, ...
    'XLim', [750 2250], ...
    'XTickLabel', [], ...
    'TickDir', 'out', ...
    'Layer', 'top');
hold on;

% Plot measured power spectrum (low-resolution)
bar(cfHz, DFTm, ...
    'LineWidth', scale, ...
    'BarWidth', 1, ...
    'FaceColor', areacolour, ...
    'EdgeColor', 'none');

% Plot analytical power spectrum (low-resolution)
bar(cfHz, aDFTm, ...
    'LineWidth', scale, ...
    'BarWidth', 1, ...
    'FaceColor', 'none', ...
    'EdgeColor', 'k');

% Set y-ticks to limits only
set(gca, 'YTick', ylim);

% Add annotation B
annotation( ...
    'textbox', ...
    'Units', 'inches', ...
    'Position', scale*[0 0.8+3*ygap 0.2 0.2], ...
    'String', 'B', ...
    'FontSize', 12*scale, ...
    'FontWeight', 'bold', ...
    'LineStyle', 'none');

% Add text description
annotation( ...
    'textbox', ...
    'Units', 'inches', ...
    'Position', scale*[2.4 0.5+3*ygap 0.7 0.2], ...
    'String', ['Filter Output' 10 'Power'], ...
    'FontSize', 8*scale, ...
    'VerticalAlignment', 'middle', ...
    'LineStyle', 'none');

% Create axes for plotting mean ZCIH
axespos = scale*[0.55 0.4+2*0.7 1.8 0.4];
axes(...
    'Units', 'inches', ...
    'Position', axespos, ...
    'LineWidth', scale, ...
    'FontSize', 8*scale, ...
    'XLim', [750 2250], ...
    'XTickLabel', [], ...
    'TickDir', 'out', ...
    'Layer', 'top');
hold on;

% Plot mean ZCIH (empirical)
area(binsHz, ZCIHm, ...
    'FaceColor', areacolour, ...
    'EdgeColor', areacolour, ...
    'LineWidth', scale);

% Plot mean ZCIH (analytical)
plot(binsHz, aZCIHm, 'k', 'LineWidth', scale);

% Set y-ticks to limits only
set(gca, 'YTick', ylim);

% Add annotation C
annotation( ...
    'textbox', ...
    'Units', 'inches', ...
    'Position', scale*[0 0.8+2*ygap 0.2 0.2], ...
    'String', 'C', ...
    'FontSize', 12*scale, ...
    'FontWeight', 'bold', ...
    'LineStyle', 'none');

% Add text description
annotation( ...
    'textbox', ...
    'Units', 'inches', ...
    'Position', scale*[2.4 0.5+2*ygap 0.5 0.2], ...
    'String', 'ZCIH', ...
    'FontSize', 8*scale, ...
    'VerticalAlignment', 'middle', ...
    'LineStyle', 'none');

% Create axes for plotting mean ZCPA
axespos = scale*[0.55 0.4+ygap 1.8 0.4];
axes(...
    'Units', 'inches', ...
    'Position', axespos, ...
    'LineWidth', scale, ...
    'FontSize', 8*scale, ...
    'XLim', [750 2250], ...
    'XTick', [500 1000 1500 2000 2500], ...
    'XTickLabel', [], ...
    'TickDir', 'out', ...
    'Layer', 'top');
hold on;

% Plot mean ZCPA (empirical)
area(binsHz, IFRHm, ...
    'FaceColor', areacolour, ...
    'EdgeColor', areacolour, ...
    'LineWidth', scale);
plot(binsHz, aIFRHm, 'k', 'LineWidth', scale);

% Plot mean ZCPA (analytical)
set(gca, 'YTick', ylim);

% Add annotation D
annotation( ...
    'textbox', ...
    'Units', 'inches', ...
    'Position', scale*[0 0.8+ygap 0.2 0.2], ...
    'String', 'D', ...
    'FontSize', 12*scale, ...
    'FontWeight', 'bold', ...
    'LineStyle', 'none');

% Add text description
annotation( ...
    'textbox', ...
    'Units', 'inches', ...
    'Position', scale*[2.4 0.5+ygap 0.5 0.2], ...
    'String', 'IFRH', ...
    'FontSize', 8*scale, ...
    'VerticalAlignment', 'middle', ...
    'LineStyle', 'none');

% Create axes for plotting mean ZCPA
axespos = scale*[0.55 0.4 1.8 0.4];
axes(...
    'Units', 'inches', ...
    'Position', axespos, ...
    'LineWidth', scale, ...
    'FontSize', 8*scale, ...
    'XLim', [750 2250], ...
    'XTick', [500 1000 1500 2000 2500], ...
    'TickDir', 'out', ...
    'Layer', 'top');
hold on;

% Plot mean ZCPA (empirical)
area(binsHz, ZCPAm, ...
    'FaceColor', areacolour, ...
    'EdgeColor', areacolour, ...
    'LineWidth', scale);
plot(binsHz, aZCPAm, 'k', 'LineWidth', scale);

% Plot mean ZCPA (analytical)
set(gca, 'YTick', ylim);

% Set y-ticks to limits only
xlabel 'Frequency (Hz)'

% Add annotation E
annotation( ...
    'textbox', ...
    'Units', 'inches', ...
    'Position', scale*[0 0.8 0.2 0.2], ...
    'String', 'E', ...
    'FontSize', 12*scale, ...
    'FontWeight', 'bold', ...
    'LineStyle', 'none');

% Add text description
annotation( ...
    'textbox', ...
    'Units', 'inches', ...
    'Position', scale*[2.4 0.5 0.5 0.2], ...
    'String', 'ZCPA', ...
    'FontSize', 8*scale, ...
    'VerticalAlignment', 'middle', ...
    'LineStyle', 'none');

% end of file %
