% MAKE_FIGURE7AND8_DATA Make Data Set for Figures 7 and 8 for JASA paper.
%
% Author: Robert Mill <rob.mill.uk@gmail.com>
% (C)2015 University of Sheffield
% Part of: (MAKE) materials for JASA paper.


% Set sample rate
fsHz = 32768;

% Construct filter models
Mhs = mgm_create(1, 100, 1/100, 0, 2*pi*1850, 0, 'real');
Mhs = mgm_add(Mhs, mgm_create(1, 1500, 1/3000, 0, 2*pi*1300, 0, 'real'));
Mhs = mgm_add(Mhs, mgm_create(1, 500, 1/1000, 0, 2*pi*1700, 0, 'real'));

% Construct signal power spectrum model
MSs = mgm_abs2(mgm_fourier(Mhs));
MSs = mgm_simplify(MSs);

% Sample impulse response
hs = mgm_impulse(Mhs, fsHz);
hs = hs(:)/fsHz;

% Number of seconds of signal and signal amplitude
nsec = 10;
sinA = 25;

% Set time steps
tsec = (0:nsec*fsHz-1)'/fsHz;

% Create a sigmoid as signal
sigmoid = 1./(1 + exp(-3*(tsec(tsec >= 3 & tsec <= 7)-5)));
sigmoid = sigmoid - min(sigmoid);
sigmoid = sigmoid / max(sigmoid);

% Sweep instantaneous frequency upward between 3 and 7 seconds
ifrHz = zeros(size(tsec));
ifrHz(tsec < 3) = 1500;
ifrHz(tsec > 7) = 1815;
ifrHz(tsec >= 3 & tsec <= 7) = 1500 + 315*sigmoid;

% Instantaneous phase
phi = cumsum(ifrHz)*2*pi/fsHz;

% Create signal and noise
rng(141282, 'v4');
n = sqrt(fsHz)*randn(nsec * fsHz, 1);
s = sinA * cos(phi);
x = s + n;
y = filter(hs, 1, x);

% DFT parameters
ndft  = 1024;
cell1 = 32;
celln = 33;
bwHz  = 64;
binw  = 4;

% Set sliding window DFT parameters
swopt              = swdft_opt;
swopt.n_fs_Hz      = int32(fsHz);
swopt.n_dft_len    = int32(ndft);
swopt.n_cell_start = int32(cell1);
swopt.n_cell_count = int32(celln);
swopt.d_bw_Hz      = bwHz;

% Adjust sliding window DFT window
swopt = swdft_window(swopt, 'gauss');
swopt = swdft_fringe(swopt);

% Get impulse response of filters
Mfb = swdft_mgm(swopt);

% Get centre frequencies of cells
cfHz  = (cell1+(0:celln-1)) * fsHz/ndft;

% Set bin frequencies
fqHz  = min(cfHz)-bwHz/2:binw:max(cfHz)+bwHz/2;
nbins = length(fqHz);

% Set ZCPA parameters 
zcopt              = zcpa_opt;
zcopt.n_channels   = int32(celln);
zcopt.n_bins       = int32(nbins);
zcopt.n_ass_mode   = int32(0);
zcopt.n_peak_mode  = int32(0);
zcopt.d_freq_start = fqHz(1);
zcopt.d_freq_width = binw;
zcopt.dp_taus_s    = 0.1*ones(nbins, 1);
zcopt.n_out_wait   = int32(1024);

map = struct( ...
	'n_bin_start', [], ...
    'n_bin_count', [], ...
    'n_zci_count', [], ...
    'd_zci_start', [], ...
    'd_zci_width', [], ...
    'dp_pdf', []);

% Set adjusted ZCPA parameters 
rzcopt              = zcpa_opt;
rzcopt.n_channels   = int32(celln);
rzcopt.n_bins       = int32(nbins);
rzcopt.n_ass_mode   = int32(1);
rzcopt.n_peak_mode  = int32(0);
rzcopt.d_freq_start = fqHz(1);
rzcopt.d_freq_width = binw;
rzcopt.dp_taus_s    = 0.1*ones(nbins, 1);
rzcopt.n_out_wait   = int32(1024);
rzcopt.sp_maps      = repmat(map, celln, 1);

% Start progress meter
fprintf('[Progress out of %d]: ', swopt.n_cell_count);

% For each DFT cell:
for n = 1:swopt.n_cell_count

    % Print progress marker
    fprintf('.');

    % Get frequencies and intervals to perform reassignment over
    f0 = cfHz(n) - bwHz;
    f1 = cfHz(n) + bwHz;
    i0 = 0.5/(cfHz(n) + bwHz);
    i1 = 0.5/(cfHz(n) - bwHz);

    % Sampling resolution of intervals
    is = i0:5e-7:i1;

    % Mask out frequencies of relevance
    mask = fqHz > f0 & fqHz <= f1;
    ix   = find(mask, 1, 'first') - 1;

    % Get magnitude response of filters
    MSa = mgm_abs2(mgm_fourier(mgm_real(Mfb{n})));
    MSa = mgm_simplify(MSa);
    map = zci_remap(mgm_multiply(MSa, MSs), sinA, is, MSa, fqHz(mask));

    % Set maps for reassigned ZCPA
    rzcopt.sp_maps(n).n_bin_start = int32(ix);
    rzcopt.sp_maps(n).n_bin_count = int32(sum(mask));
    rzcopt.sp_maps(n).n_zci_count = int32(length(is));
    rzcopt.sp_maps(n).d_zci_start = i0;
    rzcopt.sp_maps(n).d_zci_width = 5e-7;
    rzcopt.sp_maps(n).dp_pdf      = map * diag(1./sum(map));

end

% Finished progress
fprintf('OK\n');

% Compute DFT, ZCIH and reassigned ZCIH
DFT   = swdft_run(y, swopt);
ZCIH  = zcpa_run(real(DFT), zcopt);
RZCIH = zcpa_run(real(DFT), rzcopt);

% Save data for figure production
save(['data' filesep 'figure7and8_data.mat'],'DFT','ZCIH','RZCIH','fqHz','ifrHz','tsec','Mfb','MSs','cfHz',...
    'swopt','zcopt','rzcopt');

% end of file %
