function M = mgm_create(msize, A, sig, mu, w, phi, option)
% MGM_CREATE Create a modulated Gaussian mixture.
%    MGM_CREATE creates a single-component MGM with fields A = 1, SIG = 1,
%    MU = 0, W = 0, PHI = 0; that is, a real Gaussian of unit height and
%    standard deviation, centred on the origin.
%
%    MGM_CREATE(MSIZE) creates an MGM with identical components and size
%    specified by the two-element vector MSIZE. If MSIZE is a unit vector,
%    a square matrix of components is returned. The default values of each
%    component are those specified above.
%
%    MGM_CREATE(MSIZE, A, SIG, MU, W, PHI) initialises the fields A, SIG,
%    MU, W and PHI of all components. If any of these fields are the empty
%    matrix, the default value is used (see above).
%
%    MGM_CREATE(MSIZE, A, SIG, MU, W, PHI, 'symmetric') returns a version
%    of the MGM which is symmetric around zero, by adding a time- or
%    frequency-reversed version of the MGM to itself. It computes the
%    reversed version using MGM_REVERSE.
%
%    MGM_CREATE(MSIZE, A, SIG, MU, W, PHI, 'real') returns only the real
%    ('cosine') part of the MGM. It does this by calling MGM_REAL as the
%    last step.
%
%    MGM_CREATE(MSIZE, A, SIG, MU, W, PHI, 'real') returns only the
%    imaginary ('sine') part of the MGM. It does this by calling MGM_IMAG
%    as the last step.
%
%    Note that calling with the option 'symmetric, 'real' or 'imag' returns
%    a matrix twice as wide.
%
% Author: Robert Mill <rob.mill.uk@gmail.com>
% (C)2015 University of Sheffield
% Part of: (M)odulated (G)aussian (M)ixtures
%
% See also MGM_EVAL.


% Use defaults?
if nargin < 1, msize = 1; end
if nargin < 2 || isempty(A), A = 1; end;
if nargin < 3 || isempty(sig), sig = 1; end;
if nargin < 4 || isempty(mu), mu = 0; end;
if nargin < 5 || isempty(w), w = 0; end;
if nargin < 6 || isempty(phi), phi = 0; end;
if nargin < 7, option = ''; end

% Convert scalar size to a square matrix
if isscalar(msize), msize = [msize msize]; end

% Create structure and replicate to desired size
M = struct('A', A, 'sig', sig, 'mu', mu, 'w', w, 'phi', phi);
M = repmat(M, msize);

% Specify additional options:
switch option

    case 'symmetric'

        % Create a symmetric (time-/frequency-reversed) component
        M = mgm_add(M, mgm_reverse(M));

    case 'real'

        % Return only the real part
        M = mgm_real(M);

    case 'imag'

        % Return only the imaginary part
        M = mgm_imag(M);

end

% end of file %
