function ys = mgm_eval(Mx, xs, option)
% MGM_EVAL Evaluate an MGM at specific points.
%    MGM_EVAL(MX, XS) returns the value of MX at the points specified in
%    XS. The result has the same dimensions as X. The value of a single
%    component of an MGM evaluated at 'x' is
%
%       MGM(x) = [A]exp( -(x-[mu])^2/[sig]/2 + i[w]x + i[phi] ).
%
%    In general, complex values are returned.
% 
%    MGM_EVAL(MX, XS, 'real') returns the real value only. This flag should
%    be used if the result ought to be real, because small imaginary values
%    may otherwise accumulate due to rounding errors.
%
%    MGM_EVAL(MX, XS, 'imag') returns the imaginary value only.
%
% Author: Robert Mill <rob.mill.uk@gmail.com>
% (C)2015 University of Sheffield
% Part of: (M)odulated (G)aussian (M)ixtures
%
% See also MGM_EVALD, MGM_EVALD2.


% Reserve space for output
ys = zeros(size(xs));

% For each component n in Mx:
for n = 1:numel(Mx)

    % Get component
    m  = Mx(n);

    % Evaluate function
    ys = ys + m.A*exp(1i*(m.phi + m.w*xs) - (xs - m.mu).^2/m.sig^2/2);

end

% Enforce real or imaginary output?
if nargin > 2 && strcmpi(option, 'real'), ys = real(ys); end
if nargin > 2 && strcmpi(option, 'imag'), ys = imag(ys); end

% end of file %
