function Mfb = mgm_fbank_create(cfHz, bwHz, pkas, phas, option)
% MGM_FBANK_CREATE Create a filter bank of MGM impulse responses.
%    MGM_FBANK_CREATE(CFHZ, BWHZ) returns a cell array of models of complex
%    impulse responses for filters with centres CFHZ and 3dB (power)
%    bandwidths BWHZ. The envelope and fine structure are aligned at zero.
%
%    MGM_FBANK_CREATE(CFHZ, BWHZ. PKAS, PHAS) returns complex impulse
%    responses and aligns the envelope peaks at time PKAS and the zero
%    phase of the fine structure at PHAS.
%
%    MGM_FBANK_CREATE(CFHZ, BWHZ. PKAS, PHAS, 'real') returns a cell array
%    of real impulse responses.
%
%    MGM_FBANK_CREATE(CFHZ, BWHZ. PKAS, PHAS, 'imag') returns a cell array
%    of imaginary impulse responses. (That is, the quadrature impulse
%    response.)
%
%    MGM_FBANK_CREATE(CFHZ, BWHZ. PKAS, PHAS, 'array') returns complex
%    impulse responses in a structure array, rather than a cell array.
%    (This will be interpreted as the sum of impulse responses if used
%    naively. The caller should separate the responses themselves.)
%
% Author: Robert Mill <rob.mill.uk@gmail.com>
% (C)2015 University of Sheffield
% Part of: (M)odulated (G)aussian (M)ixtures
%
% See also MGM_FBANK_AUTOCOV.


% Use defaults?
if nargin < 3 || isempty(pkas), pkas = 0; end
if nargin < 4 || isempty(phas), phas = 0; end
if nargin < 5 || isempty(option), option = 'cell'; end

% Replicate data if necessary
if isscalar(cfHz), cfHz = repmat(cfHz, size(bwHz)); end
if isscalar(bwHz), bwHz = repmat(bwHz, size(cfHz)); end
if isscalar(pkas), pkas = repmat(pkas, size(cfHz)); end
if isscalar(phas), phas = repmat(phas, size(cfHz)); end

% Get sigmas corresponding to bandwidths
sigs = mgm_bw2sig(bwHz, 3, 'time') / sqrt(2);

% Make filters for each channel
Mfb = mgm_create([numel(cfHz), 1]);

% For each channel n:
for n = 1:numel(Mfb)

    % Configure inpulse response of filter
    Mfb(n).mu  = pkas(n);
    Mfb(n).w   = cfHz(n) * 2*pi;
    Mfb(n).sig = sigs(n);
    Mfb(n).phi = mod(-Mfb(n).w * phas(n), 2*pi);

end

% Post-processing options?
if strcmpi(option, 'real'), Mfb = mgm_real(Mfb, 'grid'); end
if strcmpi(option, 'imag'), Mfb = mgm_imag(Mfb, 'grid'); end

% Return result as cell not array
if ~strcmpi(option, 'array'), Mfb = num2cell(Mfb, 2); end

% end of file %
