function [ys, ts] = mgm_impulse(Mx, fs, align, cutoff, option)
% MGM_IMPULSE Evaluate an appropriate inpulse response for an MGM.
%    MGM_IMPULSE(MX) returns the impulse response associated with the MGM
%    given as MX as a row vector.
%
%    [YS, TS] = MGM_IMPULSE(...) returns the time steps associated with the
%    impulse response in TS.
%
%    MGM_IMPULSE(MX, FS) computes the times according to the sample rate
%    given in FS (specified in Hertz).
%
%    MGM_IMPULSE(MX, FS, ALIGN) aligns the response to that one sample in
%    TS is exactly aligned to ALIGN. (By default ALIGN = 1). If ALIGN is
%    either 'low' or 'high', then the first or last sample is TS is
%    aligned to a sample time.
%
%    MGM_IMPULSE(MX, FS, ALIGN, CUTOFF) specifies the number of standard
%    deviations up to which to evaluate a component. That is, the impulse
%    response is truncated at the point where all components have reached
%    CUTOFF standard deviations away from their centre. By default,
%    CUTOFF = 3 stds.
%
%    MGM_IMPULSE(MX, FS, ALIGN, CUTOFF, 'real') evaluates only the real
%    component of the output.
%
%    MGM_IMPULSE(MX, FS, ALIGN, CUTOFF, 'imag') evaluates only the
%    imaginary component of the output.
%
% Author: Robert Mill <rob.mill.uk@gmail.com>
% (C)2015 University of Sheffield
% Part of: (M)odulated (G)aussian (M)ixtures
%
% See also MGM_EVAL.


% Use defaults?
if nargin < 2 || isempty(fs), fs = 1; end
if nargin < 3 || isempty(align), align = 0; end
if nargin < 4 || isempty(cutoff), cutoff = 3; end
if nargin < 5 || isempty(option), option = []; end

% Get the lower and upper limits of the impulse response
lo = min([Mx.mu] - cutoff*[Mx.sig]);
hi = max([Mx.mu] + cutoff*[Mx.sig]);

% Align first or last sample in impulse response
if strcmpi(align, 'low'), align = lo; end
if strcmpi(align, 'high'), align = hi; end

% Compute time steps and evaluate
ts = (ceil((lo - align)*fs):floor((hi - align)*fs))/fs + align;
ys = mgm_eval(Mx, ts, option);

% end of file %
