function Mz = mgm_multiply(Mx, My, option)
% MGM_MULTIPLY Multiply two modulated Gaussian mixtures.
%    MGM_MULTIPLY(MX, MY) multiplies the two signals encoded by MX and MY
%    and returns the result as another mixture. By default, the 'grid'
%    option is used (see below).
%
%    MGM_MULTIPLY(MX, MY, 'linear') returns the mixture with its components
%    in column format.
%
%    MGM_MULTIPLY(MX, MY, 'simplify') returns a canonical form of the
%    mixture by calling MGM_SIMPLIFY as the last step. For more details,
%    see MGM_SIMPLIFY.
%
%    MGM_MULTIPLY(MX, MY, 'grid') returns the mixture with its components
%    organised into a matrix, such that the (i,j)th entry is the product of
%    the ith component of MX and the jth component of MY.
%
% Author: Robert Mill <rob.mill.uk@gmail.com>
% (C)2015 University of Sheffield
% Part of: (M)odulated (G)aussian (M)ixtures
%
% See also MGM_ADD, MGM_CONV, MGM_SIMPLIFY.


% Store results in a column by default
if nargin < 3, option = ''; end

% Create a 2D grid of blank MGMs for output
Mz = mgm_create([numel(Mx), numel(My)]);
mz = mgm_create(1);

% For each nx:
for nx = 1:numel(Mx)

    % For each ny:
    for ny = 1:numel(My)

        % Get components
        mx = Mx(nx);
        my = My(ny);

        % Pre-compute expressions
        c1 = mx.sig^2 + my.sig^2;
        c2 = (mx.sig./my.sig)^2;

        % Protect against NaNs
        if isnan(c2), c2 = 1; end

        % Store component product
        mz.A   = mx.A * my.A * exp(-(mx.mu - my.mu)^2/c1/2);
        mz.sig = 1/sqrt(1/mx.sig^2 + 1/my.sig^2);
        mz.mu  = my.mu/(1 + 1/c2) + mx.mu/(1 + c2);
        mz.w   = mx.w + my.w;
        mz.phi = mx.phi + my.phi;

        % Store component component as result
        Mz(nx,ny) = mz;

    end

end

% Post-processing options?
if strcmpi(option, 'linear'), Mz = Mz(:); end
if strcmpi(option, 'simplify'), Mz = mgm_simplify(Mz); end

% end of file %
