function Mz = mgm_real(Mx, option)
% MGM_REAL Compute the real part of an MGM.
%    MGM_REAL(MX) returns the real part of MX. This computation is carried
%    out component-wise using the Euler formula:
%
%       Re(X) = |A|cos(phi) = |A|exp(+i*arg)/2 + |A|exp(-i*arg)/2.
%                             \______________/   \______________/
%                              term1              term2
%
%    By default (or specifying the 'grid') option, the result is returned
%    as a matrix twice as wide as the input, where the components
%    corresponding to 'term1' and 'term2' are stored in the left and right
%    halves, respectively.
%
%    MGM_REAL(MX, 'simplify') returns a canonical form of the mixture by
%    calling MGM_SIMPLIFY as the last step. For more details, see
%    MGM_SIMPLIFY.
%
%    MGM_REAL(MX, 'linear') returns the mixture with its components in
%    column format.
%
% Author: Robert Mill <rob.mill.uk@gmail.com>
% (C)2015 University of Sheffield
% Part of: (M)odulated (G)aussian (M)ixtures
%
% See also MGM_IMAG, MGM_ABS2, MGM_SIMPLIFY.


% Use defaults?
if nargin < 2, option = 'grid'; end

% Copy to make 'positive' and 'negative' components
Mzn = Mx;
Mzp = Mx;

% For each component n:
for n = 1:numel(Mx)
    
    % Get component
    mx = Mx(n);
    
    % Set component values
    Mzp(n).A   = mx.A / 2;
    Mzn(n).A   = mx.A / 2;
    Mzn(n).phi = mod(-mx.phi, 2*pi);
    Mzn(n).w   = -mx.w;
    
end

% Combine positive and negative halfs into a single matrix
Mz = [Mzp Mzn];

% Post-processing options
if strcmpi(option, 'linear'), Mz = Mz(:); end
if strcmpi(option, 'simplify'), Mz = mgm_simplify(Mz); end

% end of file %
