function Mz = mgm_simplify(Mx, Acut)
% MGM_SIMPLIFY Put an MGM in a simplified canonical form.
%    MGM_SIMPLIFY(MX) simplifies MX by combining components that have the
%    same values for C, mu and w. The components are added together to
%    produce a single component at one magnitude and phase. Negative
%    numbers are encoded as phase shifts of pi. If components with A < eps
%    appear, they are removed.
%
%    MGM_SIMPLIFY(MX, ACUT), after simplifying, removes any components with
%    magnitude A smaller than ACUT.
%
% Author: Robert Mill <rob.mill.uk@gmail.com>
% (C)2015 University of Sheffield
% Part of: (M)odulated (G)aussian (M)ixtures
%
% See also EPS.


% Use defaults?
if nargin < 2 || isempty(Acut), Acut = eps; end

% Copy original into a column
Mx = Mx(:);

% Get distinct components
[dc, ~, ixs] = unique([[Mx.sig]' [Mx.mu]' [Mx.w]'], 'rows');

% Create an MGM to accommodate result
Mz = mgm_create([length(dc), 1]);

% For each distinct component n:
for n = 1:length(dc)

    % Mask out components from family
    mask = ixs == n;

    % Collect magnitudes and angles
    A      = [Mx(mask).A];
    phi    = [Mx(mask).phi];
    phasor = sum(A .* exp(1i*phi));

    % Put results into component
    Mz(n).A   = abs(phasor);
    Mz(n).sig = dc(n,1);
    Mz(n).mu  = dc(n,2);
    Mz(n).w   = dc(n,3);
    Mz(n).phi = angle(phasor);

end

% Delete components that are negligible
mask     = [Mz.A]./[Mz.sig] <= Acut;
Mz(mask) = [];

% end of file %
