function [swopt, win] = swdft_window(swopt, wintype, arg1, arg2)
% SWDFT_WINDOW Set the window parameters for an SWDFT.
%    SWDFT_WINDOW(SWOPT, 'none') returns parameters that correspond to no
%    windowing for the DFT. The result is a new parameter set with fields
%    filled in.
%
%    SWDFT_WINDOW(SWOPT, 'gauss', BWHZ) returns a window to given a 3-dB
%    (power) bandwidth of BWHZ.
%
%    SWDFT_WINDOW(SWOPT, 'gauss', BWHZ, N) returns a window to given an
%    N-dB (power) bandwidth of BWHZ.
%
%    SWDFT_WINDOW(SWOPT, 'user', WIN, WIDTH) uses the user window WIN,
%    specified in the time domain. The frequency domain window is 2*WIDTH+1
%    cells wide.
%
%    [SWOPT, WIN] = SWDFT_WINDOW(...) returns the time domain window in
%    WIN.
%
% Author: Robert Mill <rob.mill.uk@gmail.com>
% (C)2015 University of Sheffield
% Part of: (S)liding (W)indow (D)iscrete (F)ourier (T)ransform
%
% See also SWDFT_RUN, SWDFT_OPT, SWDFT_FRINGE.


% Get sample rate and length of DFT
fsHz = double(swopt.n_fs_Hz);
ndft = double(swopt.n_dft_len);

% Depending on the window type:
switch wintype

    case 'user'

        % Treat argument one as the window itself (in time domain)
        win = arg1;

        % Set the window
        W = fft(win, ndft)/ndft;
        W = W([end-arg2+1:end, 1:arg2+1]);

    case 'gauss'

        % Set bandwidth and fidelity of window
        if nargin < 3 || isempty(arg1), arg1 = swopt.d_bw_Hz; end
        if nargin < 4 || isempty(arg2), arg2 = 3; end

        % Rename arguments (for ease of reading)
        swopt.d_bw_Hz = arg1;
        dB            = arg2;

        % Make impulse response in time domain
        ts  = ((0:ndft-1) - ndft/2 + 1/2)/fsHz;
        sig = sqrt(log(10^(dB/10)))/(pi*swopt.d_bw_Hz);
        win = exp(-ts.^2/sig^2/2);

        % Convert window to frequency domain
        W  = fft(win)/ndft;
        ix = find(abs(W)/max(abs(W)) < 0.01, 1);
        W  = W([end-ix+2:end, 1:ix]);

    case 'none'

        % No window
        swopt.d_bw_Hz = ndft/fsHz;

        % Use window of ones
        win = ones(1, ndft);
        W   = complex(1);

    otherwise

        % Generate an error
        error('Unknown window type given to swdft_window.')

end

% If the window is real by this point, ensure it has an imaginary part
if isreal(W), W = complex(W); end

% Set window field
swopt.dc_f_win = W;       

% end of file %
