function I = util_trapz2(S, xs, ys, mask)
% UTIL_TRAPZ2 Trapezoidal integration in two dimensions.
%    UTIL_TRAPZ2(S, XS, YS) returns the volume under the surface S samples
%    at x positions XS and y positions YS (which need not be uniformly
%    spaced).
%
%    UTIL_TRAPZ2(S, XS, YS, MASK) returns the volume under the surface S
%    only at those points where MASK is true. MASK should be a logical
%    matrix the same size as S.
%
% Author: Robert Mill <rob.mill.uk@gmail.com>
% (C)2015 University of Sheffield
% Part of: (UTIL)ity functions for the JASA paper


% Use defaults?
if nargin < 4, mask = []; end

% Is there a mask?
if ~isempty(mask)

    % Apply NaNs where there is no mask
    S(~mask) = NaN;

end

% Average neighbours
Ss = convn(S, ones(2)/4);
Ss = Ss(2:end-1,2:end-1);

% Compute differences measured over
dxs = diff(xs(:))';
dys = diff(ys(:));

% Compute volumes of trapezoids and them sum all those which aren't NaN
els = bsxfun(@times, dxs, dys) .* Ss;
I   = sum(els(~isnan(els(:))));

% end of file %
