function iapdf = zci_pk_pdf(Mga, is, as, option)
% ZCI_PK_PDF Zero crossing interval probability density function.
%    ZCI_PK_PDF(MGA, IS, AS) returns the probability density associated
%    with observing the intervals IS (seconds) paired with the squared peak
%    values AS (arb. units) given the model autocovariance function MGA.
%    This is equivalent to invoking ZCI_PK_PDF(MGA, IS, AS, 'paired').
%
%    ZCI_PK_PDF(MGA, IS, AS, 'grid') returns the probability density
%    function for the intervals and squared peaks in IS and AS
%    respectively, forming a grid by pairing every IS with every AS. (In
%    other words, it returns the 2d joint density function.) The intervals
%    are indexed in rows, and the squared peaks in columns.
%
%    ZCI_PK_PDF(MGA, IS, [], 'mean') returns the expected squared-peak
%    value given the intervals listed in IS (seconds).
% 
% Author: Robert Mill <rob.mill.uk@gmail.com>
% (C)2015 University of Sheffield
% Part of: (Z)ero (C)rossing (I)ntervals
%
% See also ZCI_PDF, ZCI_SIN_PK_HIST.


% Use defaults?
if nargin < 4 || isempty(option), option = 'paired'; end

% Turn intervals into a column
is = is(:);

% Evaluate the autocovariance function at three points.
ga0 = mgm_eval(Mga, 0, 'real');
ga1 = mgm_eval(Mga, is/2, 'real');
ga2 = mgm_eval(Mga, is, 'real');

% Compute intermediate value
beta = (ga0.^2 + ga0.*ga2 - 2*ga1.^2)./(ga0 + ga2);

% Return mean square mid-point?
switch option
    
    case 'mean'

        % Return mean square mid-point
        iapdf = 3 * beta';

    case 'grid'

        % Normalise autocovariance to get autocorrelation coefficient
        Mrho = mgm_norm(Mga, 'zero');

        % Evaluate autocorrelation coefficient and derivatives.
        rho   = mgm_eval(Mrho, is);
        rhod  = mgm_evald(Mrho, is);
        rhod2 = mgm_evald2(Mrho, is);

        % Compute numerator and denominator of interval p.d.f.
        A      = rho.^2 - 1;
        ipdf_n = rho .* rhod.^2 - A .* rhod2;
        ipdf_d = 2 * sqrt(mgm_evald2(Mrho, 0) * A.^3);

        % Compute interval p.d.f.
        ipdf = real(ipdf_n ./ ipdf_d);

        % Reshape matrices
        beta = repmat(beta, 1, numel(as));
        ipdf = repmat(ipdf, 1, numel(as));
        as   = repmat(as(:), 1, numel(is))';

        % Compute joint interval/peak p.d.f.
        iapdf = sqrt(as./(2*pi*beta.^3)) .* exp(-as./beta/2) .* ipdf;

    case 'paired'

        % Normalise autocovariance to get autocorrelation coefficient
        Mrho = mgm_norm(Mga, 'zero');

        % Evaluate autocorrelation coefficient and derivatives.
        rho   = mgm_eval(Mrho, is);
        rhod  = mgm_evald(Mrho, is);
        rhod2 = mgm_evald2(Mrho, is);

        % Compute numerator and denominator of interval p.d.f.
        A      = rho.^2 - 1;
        ipdf_n = rho .* rhod.^2 - A .* rhod2;
        ipdf_d = 2 * sqrt(mgm_evald2(Mrho, 0) * A.^3);

        % Compute interval p.d.f.
        ipdf = real(ipdf_n ./ ipdf_d);
        
        % Compute joint interval/peak p.d.f.
        iapdf = sqrt(as./(2*pi*beta.^3)) .* exp(-as./beta/2) .* ipdf;

end

% end of file %
