function [map, fqHz] = zci_remap(MSn, A, is, MHa, fqHz)
% ZCI_REMAP Re-map frequencies in channel according to noise model.
%    ZCI_REMAP(MSN, A, IS, MHA, FQHZ) generates probability density
%    functions for the zero crossing intervals IS (seconds) conditioned on
%    the presence of a sinusoid of amplitude A at frequencies FQHZ (Hertz)
%    against a noise background with power-spectral density model MSN if
%    the sinusoid has been attenuated by the analysis filter with model
%    squared magnitude respcnse MHA.
%
%    ZCI_REMAP(MSN, A, IS, MHA) automatically chooses a series of
%    frequencies suitable to the intervals expressed in IS. Namely, 100
%    frequencies in the range 1/2/max(is) to 1/2/min(is).
% 
% Author: Robert Mill <rob.mill.uk@gmail.com>
% (C)2015 University of Sheffield
% Part of: (Z)ero (C)rossing (I)ntervals
%
% See also ZCI_SIN_PDF, MAKE_FIGURE7AND8_DATA.


% Use defaults?
if nargin < 4 || isempty(MHa), MHa = MSn; end
if nargin < 5, fqHz = []; end

% Use default frequencies?
if isempty(fqHz)

    % Space 100 frequencies uniformly based on min and max intervals
    fq0  = 1/2/max(is);
    fq1  = 1/2/min(is);
    fqHz = linspace(fq0, fq1, 100);

end

% Get autocovariance from inverse Fourier transform of noise spectrum
Mga = mgm_ifourier(MSn);

% Reserve space for map
map = zeros(length(fqHz), length(is));

% For each frequency n:
for n = 1:length(fqHz)

    % Get attenuation due to the filter
    attn = mgm_eval(MHa, 2*pi*fqHz(n), 'real');

    % Compute distribution of sine in noise and store in row
    map(n,:) = zci_sin_pdf(Mga, sqrt(attn)*A, fqHz(n), is);

end

% end of file %
