function zcih = zci_sin_hist(cfg, N, bins, echo)
% ZCI_SIN_HIST Histogram of zero crossing intervals.
%    ZCI_SIN_HIST(CFG, N, BINS) manufactures a signal according to the
%    specification in CFG of length N (samples) and then collects the zero
%    crossings intervals into the histogram with bin centres BINS
%    (seconds).
%
%    The signal is created using ZCI_SIN_STATS. For a description of what
%    is required in the CFG structure, see documentation for ZCI_SIN_STATS.
%
%    ZCI_SIN_HIST(CFG, N, BINS, 'echo') echoes the progress every 1,000,000
%    samples that are processed.
% 
% Author: Robert Mill <rob.mill.uk@gmail.com>
% (C)2015 University of Sheffield
% Part of: (Z)ero (C)rossing (I)ntervals
%
% See also ZCI_CDF, ZCI_PDF, ZCI_SIN_STATS.


% Use defaults?
if nargin < 4 || isempty(echo), echo = 'off'; end

% Reserve space for histogram
zcih = zeros(size(bins));

% Initial state
st = [];

% While there are more samples to process:
while N > 0

    % Compute size of segment and then generate random signal
    Nseg      = min(N, 1000000);
    [zci, st] = zci_sin_stats(cfg, Nseg, st);

    % Store intervals in histogram
    zcih = zcih + hist(zci, bins);

    % Decrease number of samples left to process
    N = N - Nseg;

    % Echo progress?
    if strcmpi(echo, 'echo')

        % Print progress to console
        fprintf('zci_sin_hist has %d samples left to process.\n', N);

    end

end

% end of file %
