function [ipdf, etc] = zci_sin_pdf(Mga, sinA, sinfHz, is)
% ZCI_SIN_PDF Zero crossing interval p.d.f. for sine-in-noise process.
%    ZCI_SIN_PDF(MGA, SINA, SINFHZ, IS) returns the p.d.f. of the zero
%    crossing intervals a sinusoid with amplitude SINA and frequency SINFHZ
%    mixed with a Gaussian random process with autocovariance MGA, where
%    MGA is a modulated Gaussian mixture. The intervals are given in IS, in
%    seconds.
%
%    [IPDF, ETC] = ZCI_SIN_PDF(...) returns the interval p.d.f. in IPDF and
%    in ETC returns auxilliary data as a structure with fields
%       .x   - points to evaluate Rayl. components/pulse at
%       .Y   - matrix of Rayleigh components (in columns)
%       .tgt - target pulse
%       .w   - weights applied to Rayleigh components
% 
% Author: Robert Mill <rob.mill.uk@gmail.com>
% Date: August 2013
% Part of: (Z)ero (C)rossing (I)ntervals
%
% See also ZCI_SIN_HIST.


% Set points to evaluate Rayleigh component at
c = logspace(-2, 0.3, 50);
x = 0:0.001:10;

% Reserve space for Rayleigh components
Y = zeros(numel(x), numel(c));

% For each Rayleigh component n:
for n = 1:length(c)

    % Create Rayleigh component
    Y(:,n) = x/c(n)^2.*exp(-x.^2/c(n)^2/2);

end

% Compute weights for component p.d.f.s using pseudoinverse
tgt = 1/sqrt(2*pi*0.01)*exp(-(x-1).^2/(2*0.01))';
w   = pinv(Y) * tgt;

% Reserve space for interval p.d.f.
ipdf = zeros(size(is));

% Create MGM for pure cosine
Mcos = mgm_cos(sinA^2, 2*pi*sinfHz);

% For each component n:
for n = 1:numel(c)

    % Generate autocorrelation function
    Mrho = mgm_add(Mga, mgm_scale(Mcos, c(n)^2));
    Mrho = mgm_norm(Mrho, 'zero');

    % Add component p.d.f.
    ipdf = ipdf + w(n)*zci_pdf(Mrho, is);

end

% Return auxilliary data
etc = struct('x', x, 'Y', Y, 'tgt', tgt, 'w', w);

% end of file %
