function iapdf = zci_sin_pk_hist(cfg, N, ibins, abins, echo)
% ZCI_SIN_PK_HIST Histogram of zero crossing intervals and squared peaks.
%    ZCI_SIN_PK_HIST(CFG, N, IBINS, ABINS) manufactures a signal according
%    to the specification in CFG of length N (samples) and then collects
%    the zero crossings intervals and peaks into the histogram with bin
%    centres IBINS (for intervals, seconds) and ABINS (for squared peaks).
%
%    The return value is a matrix with a many rows/columns as IBINS/ABINS,
%    respectively. The histogram is not normalised.
%
%    The signal is created using ZCI_SIN_STATS. For a description of what
%    is required in the CFG structure, see documentation for ZCI_SIN_STATS.
%
%    ZCI_SIN_PK_HIST(CFG, N, IBINS, ABINS, 'echo') echoes the progress
%    every 1,000,000 samples that are processed.
% 
% Author: Robert Mill <rob.mill.uk@gmail.com>
% (C)2015 University of Sheffield
% Part of: (Z)ero (C)rossing (I)ntervals
%
% See also ZCI_PK_PDF, ZCI_SIN_STATS.


% Use defaults?
if nargin < 5 || isempty(echo), echo = 'off'; end

% Reserve space for histogram
iapdf = zeros(length(ibins), length(abins));

% Initial state
st = [];

% While there are more samples to process:
while N > 0

    % Compute size of segment and sample intervals and peaks from a random
    % signal
    Nseg         = min(N, 1000000);
    [zci,st,pks] = zci_sin_stats(cfg, Nseg, st);

    % Store intervals in histogram
    iapdf = iapdf + util_hist3([zci, pks], ibins, abins);

    % Decrease number of samples left to process
    N = N - Nseg;

    % Echo progress?
    if strcmpi(echo, 'echo')

        % Print progress to console
        fprintf('zci_sin_pk_hist has %d samples left to process.\n', N);

    end

end

% end of file %
