function [iapdf, etc] = zci_sin_pk_pdf(Mga, sinA, sinfHz, is, as, option)
% ZCI_SIN_PK_PDF Zero crossing interval/peak p.d.f. for sine-in-noise process.
%    ZCI_SIN_PK_PDF(MGA, SINA, SINFHZ, IS, AS) returns the p.d.f. of the
%    zero crossing intervals and squared peaks of a sinusoid with amplitude
%    SINA and frequency SINFHZ mixed with a Gaussian random process with
%    autocovariance MGA, where MGA is a modulated Gaussian mixture. The
%    intervals are given in IS, in seconds, and are paired with squared
%    peak values given in AS. This is equivalent to invoking
%    ZCI_SIN_PK_PDF(MGA, SINA, SINFHZ, IS, AS, 'paired').
%
%    ZCI_SIN_PK_PDF(MGA, SINA, SINFHZ, IS, AS, 'grid') returns the
%    probability density function for the intervals and squared peaks in IS
%    and AS respectively, forming a grid by pairing every IS with every AS.
%    (In other words, it returns the 2d joint density function.) The
%    intervals are indexed in rows, and the squared peaks in columns.
%
%    ZCI_SIN_PK_PDF(MGA, SINA, SINFHZ, IS, [], 'mean') returns the expected
%    squared-peak value given the intervals listed in IS (seconds).
%
%    [IAPDF, ETC] = ZCI_SIN_PK_PDF(...) returns the joint interval/peak
%    p.d.f. in IAPDF and in ETC returns auxilliary data as a structure with
%    fields
%       .x   - points to evaluate Rayl. components/pulse at
%       .Y   - matrix of Rayleigh components (in columns)
%       .tgt - target pulse
%       .w   - weights applied to Rayleigh components
% 
% Author: Robert Mill <rob.mill.uk@gmail.com>
% (C)2015 University of Sheffield
% Part of: (Z)ero (C)rossing (I)ntervals
%
% See also ZCI_SIN_HIST.


% Use defaults?
if nargin < 6 || isempty(option), option = 'paired'; end

% Set points to evaluate Rayleigh component at
c = logspace(-2, 0.3, 50);
x = 0:0.001:10;

% Reserve space for Rayleigh components
Y = zeros(numel(x), numel(c));

% For each Rayleigh component n:
for n = 1:length(c)

    % Create Rayleigh component
    Y(:,n) = x/c(n)^2.*exp(-x.^2/c(n)^2/2);

end

% Compute weights for component p.d.f.s using pseudoinverse
tgt = 1/sqrt(2*pi*0.01)*exp(-(x-1).^2/(2*0.01))';
w   = pinv(Y) * tgt;

% Depending on the output format:
switch option

    case 'grid'

        % Reserve space for joint interval/peak p.d.f.
        iapdf = zeros(length(is),length(as));

    case 'paired'

        % Reserve space for interval/peak p.d.f.
        iapdf = zeros(length(is), 1);

end

% Create MGM for pure cosine
Mcos = mgm_cos(sinA^2, 2*pi*sinfHz);

% For each component n:
for n = 1:numel(c)

    % Generate autocovariance function
    Mga_cos = mgm_add(Mga, mgm_scale(Mcos, c(n)^2));

    % Add component p.d.f.
    iapdf = iapdf + w(n)*zci_pk_pdf(Mga_cos, is, as, option);

end

% Return auxilliary data
etc = struct('x', x, 'Y', Y, 'tgt', tgt, 'w', w);

% end of file %
