function res = zci_sin_pk_roc_emp(cfg, opt)
% ZCI_SIN_PK_ROC_EMP Compute a point on ROC empirically for a ZCI/PK detector.
%    ZCI_SIN_PK_ROC_EMP(CFG, OPT) returns a point on an ROC curve for a
%    sine-in-noise detection task with the configuration given in CFG. The
%    detections are based on paired measurements of zero crossing intervals
%    and peak-square amplitude values across those intervals. The options
%    for how to approximate the ROC are given in OPT. The pairs of zero
%    crossing intervals and squared peaks are derived for the noise-only
%    (H0) and sine-in-noise processes (H1) by sampling a random process of
%    the appropriate type.
%
%    CFG is a structure with the following fields (obligatory):
%       .snr - snr (dB) tone power/noise power in 1Hz band (+/- freqs.)
%       .fsHz - sample rate of process (Hz)
%       .filter.cfHz - centre frequency of the analysis filter (Hz)
%       .filter.bwHz - 3dB half-power point bandwidth of filter (Hz)
%       .signal.sinfHz - signal frequency
%
%    OPT is a structure with the following fields (with defaults given if
%    fields are omitted):
%       .eta - likelihood ratio to evaluate ROC curve at (first pass) [1]
%       .nsec - number of seconds worth of data to base ROC point on [30]
%
%    The results are given in a structure with the following fields:
%       .eta - likelihood ratio threshold (from OPT)
%       .pFA - probability of a false alarm given LR threshold eta
%       .pHit - probability of a hit given LR threshold eta
%       .cfg0 - sine-in-noise configuration for H0
%       .cfg1 - sine-in-noise configuration for H1
%       .zci0 - zero crossing intervals measured for H0
%       .zci1 - zero crossing intervals measured for H1
%       .pk0 - squared peak values measured for H0
%       .pk1 - squared peak values measured for H1
%
%   Note that .cfg0 and .cfg1 are configurations used by the sine-in-noise
%   generator should not be confused with the input parameter CFG.
% 
% Author: Robert Mill <rob.mill.uk@gmail.com>
% (C)2015 University of Sheffield
% Part of: (Z)ero (C)rossing (I)ntervals
%
% See also ZCI_SIN_PK_ROC, ZCI_SIN_ROC_EMP.


% Use defaults?
if nargin < 2 || isempty(opt), opt = []; end

% Use default options?
if ~isfield(opt, 'eta'), opt.eta = 1; end
if ~isfield(opt, 'ninterp'), opt.ninterp = 150; end
if ~isfield(opt, 'nsec'), opt.nsec = 30; end
if ~isfield(opt, 'is'),
    i0 = 1/(cfg.filter.cfHz * 2^(3/2));
    i1 = 1/(cfg.filter.cfHz * 2^(1/2));
    opt.is = i0:0.5e-7:i1;
end

% Create analysis filter
sig = mgm_bw2sig(cfg.filter.bwHz, 3, 'time') * sqrt(2);
Mha = mgm_create(1, 1, sig, 0, 2*pi*cfg.filter.cfHz, 0, 'real');
MHa = mgm_abs2(mgm_fourier(Mha));

% Compute model power spectrum of noise process
MS0 = mgm_simplify(mgm_scale(MHa, 1/2));

% Get attenuation of sine wave due to filter
attn = mgm_eval(MHa, 2*pi*cfg.signal.sinfHz, 'real');

% Set sine amplitude - then with attenuation from filter
sinA   = sqrt(2*10^(cfg.snr/10));
sinAat = sqrt(attn) * sinA;

% Compute autocovariance function
Mga0 = mgm_ifourier(MS0);

% Configure generator for noise only
cfg0 = struct( ...
    'Mha', Mha, ...
    'fsHz', cfg.fsHz, ...
    'sinfHz', 0, ...
    'sinA', 0);

% Measure statistics from randomly-generated signal
[zci0, ~, pk0] = zci_sin_stats(cfg0, opt.nsec * cfg.fsHz);

% Compute empirical probabilities of false alarm
p00 = zci_pk_pdf(Mga0, zci0, pk0, 'paired');
p10 = zci_sin_pk_pdf(Mga0, sinAat, cfg.signal.sinfHz, zci0, pk0, 'paired');
pFA = mean(p10 > opt.eta * p00);

% Configure generator for signal+noise
cfg1 = struct( ...
    'Mha', Mha, ...
    'fsHz', cfg.fsHz, ...
    'sinfHz', cfg.signal.sinfHz, ...
    'sinA', sinA);

% Measure statistics from randomly-generated signal
[zci1, ~, pk1] = zci_sin_stats(cfg1, opt.nsec * cfg.fsHz);

% Compute empirical probabilities of hit
p01  = zci_pk_pdf(Mga0, zci1, pk1, 'paired');
p11  = zci_sin_pk_pdf(Mga0, sinAat, cfg.signal.sinfHz, zci1, pk1, 'paired');
pHit = mean(p11 > opt.eta * p01);

% Create structure to return performance of interval detector
res = struct(...
    'eta', opt.eta, ...
    'pFA', pFA, ...
    'pHit', pHit, ...
    'cfg0', cfg0, ...
    'cfg1', cfg1, ...
    'zci0', zci0, ...
    'zci1', zci1, ...
    'pk0', pk0, ...
    'pk1', pk1);

% end of file %
