function res = zci_sin_roc(cfg, opt)
% ZCI_SIN_ROC Compute an analytical ROC curve for a sine-in-noise ZCI detector.
%    ZCI_SIN_ROC(CFG, OPT) returns an ROC curve for a sine-in-noise
%    detection task with the configuration given in CFG. The detections are
%    based on the zero crossing intervals of the process. The options for
%    how to approximate the ROC are given in OPT.
%
%    The ROC is derived in a two-step process. In the first step, fifty
%    likelihood ratio thresholds are spaced on a log scale from 10^-3 to
%    10^3. An ROC is derived for these choices of threshold. This gives a
%    rough ROC curve that covers the unit square. In the second step, the
%    two points that are furthest apart in ROC space (Euclidean) are found, and
%    a likelihood threshold between the two is interpolated so as to smooth
%    the ROC curve somewhat (as it appears on a linear-linear scale). This
%    is repeated iteratively, for 150 steps (by default). See OPT below.
%
%    CFG is a structure with the following fields (obligatory):
%       .snr - snr (dB) tone power/noise power in 1Hz band (+/- freqs.)
%       .filter.cfHz - centre frequency of the analysis filter (Hz)
%       .filter.bwHz - 3dB half-power point bandwidth of filter (Hz)
%       .signal.sinfHz - signal frequency
%
%    OPT is a structure with the following fields (with defaults given if
%    fields are omitted):
%       .etas - likelihood ratios to evaluate ROC curve at (first pass)
%       .ninterp - number of times to interpolate ROC curve (second pass)
%       .is - intervals to produce analytical pdfs over
%
%    The results are given in a structure with the following fields:
%       .etas - likelihood threshold for point on ROC
%       .pFA - probability of false alarms corresponding to etas
%       .pHit - probability of hits corresponding to etas
%       .isinterp - mask indicating whether LRs in etas were interpolated
%       .is - intervals over which pdfs were compared
%       .pdf0 - conditional pdf for no-signal hypothesis
%       .pdf1 - conditional pdf for signal hypothesis
% 
% Author: Robert Mill <rob.mill.uk@gmail.com>
% (C)2015 University of Sheffield
% Part of: (Z)ero (C)rossing (I)ntervals
%
% See also ZCI_SIN_ROC_EMP, ZCI_SIN_PK_ROC.


% Use defaults?
if nargin < 2 || isempty(opt), opt = []; end

% Use default options?
if ~isfield(opt, 'etas'), opt.etas = logspace(-3, 3, 50); end;
if ~isfield(opt, 'ninterp'), opt.ninterp = 150; end;
if ~isfield(opt, 'is'),
    i0 = 1/(cfg.filter.cfHz * 2^(3/2));
    i1 = 1/(cfg.filter.cfHz * 2^(1/2));
    opt.is = i0:0.5e-7:i1;
end

% Create analysis filter
sig = mgm_bw2sig(cfg.filter.bwHz, 3, 'time') * sqrt(2);
Mha = mgm_create(1, 1, sig, 0, 2*pi*cfg.filter.cfHz, 0, 'real');
MHa = mgm_abs2(mgm_fourier(Mha));

% Compute model power spectrum of noise process
MS0 = mgm_simplify(mgm_scale(MHa, 1/2));

% Get attenuation of sine wave due to filter
attn = mgm_eval(MHa, 2*pi*cfg.signal.sinfHz, 'real');

% Set sine amplitude - then with attenuation from filter
sinA   = sqrt(2*10^(cfg.snr/10));
sinAat = sqrt(attn) * sinA;

% Compute autocovariance function
Mga0  = mgm_ifourier(MS0);
Mrho0 = mgm_norm(Mga0, 'zero');

% Compute interval pdfs
pdf0 = zci_pdf(Mrho0, opt.is);
pdf1 = zci_sin_pdf(Mga0, sinAat, cfg.signal.sinfHz, opt.is);

% Compute interval integration step
di = opt.is(2) - opt.is(1);

% Reserve space for statistics
R = NaN(numel(opt.etas) + opt.ninterp, 4);

% For each value of eta n:
for n = 1:length(opt.etas)

    % Compute decision regions
    d1mask = pdf1 > pdf0 * opt.etas(n);

    % Calculate and store hit and false alarm probability
    R(n,1) = opt.etas(n);
    R(n,2) = sum(pdf0(d1mask)) * di;
    R(n,3) = sum(pdf1(d1mask)) * di;
    R(n,4) = 0;

end

% For each interpolation step n:
for n = 1:opt.ninterp

    % Get Euclean distances between points on ROC
    dR = diff(R);
    dR = sqrt(dR(:,2).^2 + dR(:,3).^2);

    % Get the points on ROC curve that are visually furthest apart
    [~,j] = max(dR);

    % Insert a new threshold position
    etanew       = mean(R([j j+1], 1));
    R(j+2:end,:) = R(j+1:end-1,:);

    % Compute decision regions
    d1mask = pdf1 > pdf0 * etanew;

    % Calculate and store hit and false alarm probability
    R(j+1,1) = etanew;
    R(j+1,2) = sum(pdf0(d1mask)) * di;
    R(j+1,3) = sum(pdf1(d1mask)) * di;
    R(j+1,4) = 1;

end

% Create structure to return performance of interval detector
res = struct(...
    'etas', R(:,1), ...
    'pFA', R(:,2), ...
    'pHit', R(:,3), ...
    'isinterp', R(:,4), ...
    'is', opt.is, ...
    'pdf0', pdf0, ...
    'pdf1', pdf1);

% end of file %
