function [zci, st, pks] = zci_sin_stats(cfg, N, st)
% ZCI_SIN_STATS Sample zero crossing intervals/peaks for sine in noise.
%    ZCI_SIN_STATS(CFG, N) generates N samples from the sine+noise process
%    described in CFG whose fields are:
%       .Mha    - model (real) impulse response
%       .fsHz   - sample rate (Hz)
%       .sinfHz - frequency of tone (Hertz)
%       .sinA   - amplitude of tone (prior to filter)
%    and returns as output is a vector of zero crossing intervals (measured
%    in seconds).
%
%    The noise passed through the filter has a uniform power spectral
%    density with unit power in a 1Hz bandwidth.
%
%    [ZCI, ST] = ZCI_SIN_STATS(CFG, N, ST) returns the state in ST as a
%    structure containing
%       .z     - filter state
%       .ha    - sampled impulse response
%       .start - number of samples processed
%       .yend  - samples carried over from previous noise
%    and receives the same state as a parameter.
% 
%    [ZCI, ST, PKS] = ZCI_SIN_STATS(...) returns the squared peaks paired
%    with the intervals ZCI in PKS.
%
% Author: Robert Mill <rob.mill.uk@gmail.com>
% (C)2015 University of Sheffield
% Part of: (Z)ero (C)rossing (I)ntervals


% Use defaults?
if nargin < 3, st = []; end

% State not supplied?
if isempty(st)

    % Create an impulse response for the analysis filter
    st.ha    = mgm_impulse(cfg.Mha, cfg.fsHz, [], [], 'real') / cfg.fsHz;
    st.start = 0;

    % Make sine plus noise
    ts = (st.start:st.start + length(st.ha) - 1)'/cfg.fsHz;
    s  = cfg.sinA*cos(2*pi*cfg.sinfHz*ts);
    n  = sqrt(cfg.fsHz/2) * randn(length(st.ha), 1);

    % Fill filter with sine plus noise
    [~,st.z] = filter(st.ha, 1, s + n);
    st.yend  = [];

end

% Compute length to filter and starting sample
st.start = st.start + N;

% Make sine plus noise
ts = (st.start:st.start + N - 1)'/cfg.fsHz;
s  = cfg.sinA*cos(2*pi*cfg.sinfHz*ts);
n  = sqrt(cfg.fsHz/2) * randn(N, 1);

% Filter and keep overlap with previous signal
[y,st.z] = filter(st.ha, 1, s + n, st.z);
yover    = [st.yend; y];

% Get indices of 1st samples in sign changes
scix = find(diff(yover < 0) ~= 0);

% Compute interpolated intervals
y0  = yover(scix);
y1  = yover(scix + 1);
zci = diff(scix + y0 ./ (y0 - y1)) / cfg.fsHz;

% More than three arguments requested?
if nargout >= 3

    % Get squared amplitude and find peaks
    sqa = yover.^2;
    pks = arrayfun(@(s,e) max(sqa(s:e)), scix(1:end-1), scix(2:end));

end

% Was there no zero crossing?
if isempty(scix)

    % If not, increase overlap
    st.yend = yover;

else

    % If so, store signal from last zero crossing
    st.yend = yover(scix(end):end);

end

% end of file %
