function ps = zcpa_mean(cfHz, bwHz, zcopt, MSin)
% ZCPA_MEAN Mean ZCPA or ZCIH frequency pseudospectrum.
%    ZCPA_MEAN(CFHZ, BWHZ, ZCOPT, MSIN) returns the mean zero crossing with
%    peak amplitudes pseudospectrum when the noise process described by the
%    mixture model MSIN (see help mgm) is passed into a filter bank with
%    Gaussian shaped pass bands centred on CFHZ and with bandwidths BWHZ.
%    The options for the zero crossing/peak processing are given in ZCOPT.
%    (See ZCPA_OPT.)
%
% Author: Robert Mill <rob.mill.uk@gmail.com>
% (C)2015 University of Sheffield
% Part of: (Z)ero (C)rossings with (P)eak (A)mplitudes
%
% See also ZCPA_OPT.


% Get multipliers associated with edges of octave band
up8v = sqrt(2);
dn8v = 1/up8v;

% Get edges of bins
nbins   = double(zcopt.n_bins);
bin1Hz  = zcopt.d_freq_start;
binwHz  = zcopt.d_freq_width;
edgesHz = double(0:nbins)*binwHz + bin1Hz - binwHz/2;

% Get autocovariance function of filter outputs
Mfb  = mgm_fbank_create(cfHz, bwHz, [], [], 'real');
Mgas = mgm_fbank_autocov(Mfb, MSin);

% Reserve space for pseudospectrum
ps = zeros(nbins + 1, 1);

% For each input channel n:
for n = 1:numel(Mgas)

    % Compute autocovariance and autocorrelation coefficient
    Mrho = mgm_norm(Mgas{n}, 'zero');

    % Get edges of octave band around the c.f.
    f0 = cfHz(n) * dn8v;
    f1 = cfHz(n) * up8v;

    % Only compute cdf in valid region
    mask = edgesHz > f0 & edgesHz < f1;

    % 
    is = 1./edgesHz(mask)/2;
    
    % Compute interval histogram
    fpdf       = zeros(size(ps));
    fpdf(mask) = [-diff(zci_cdf(Mrho, is)) 0];

    switch zcopt.n_peak_mode
        
        case 0
    
            % [do nothing]
            
        case 1
            
            % 
            isc        = is(2:end) + diff(is)/2;
            pk         = [zci_pk_pdf(Mgas{n}, isc, [], 'mean')'; 0];
            fpdf(mask) = fpdf(mask) .* pk;
            
    end

    % Add interval histogram to pseudospectrum (constant weighting)
    ps = ps + fpdf;

end

% Remove the end bin
ps(end) = [];

% end of file %
