/* mx_ifrh_run.c */

/* Code invocable from Matlab to produce instantaneous frequency histogram.
 * Author: Robert Mill <rob.mill.uk@gmail.com>
 * (C)2015 University of Sheffield
 */

/* INCLUDE HEADERS */

#include <mex.h>
#include "mx_read.h"
#include "mx_ifrh.h"


/* FUNCTION DEFINITIONS */

void mexFunction(
    int n_lhs,
    mxArray *mp_lhs[],
    int n_rhs,
    const mxArray *mp_rhs[] )
{
    int n_in_row;
    int n_in_col;
    int n_out_row;
    int n_out_col;

    /* Inputs */
    const mxArray *mp_in;
    double        *dp_in_re;
    double        *dp_in_im;

    /* Outputs */
    mxArray *mp_out;
    double  *dp_out;

    /* Memory for options and state */
    IFRH_Options sp_opt   [1];
    IFRH_State   sp_state [1];

    /* Not enough arguments? */
    if( n_rhs < 2 )
    {
        /* Exit with error message */
        mexErrMsgTxt( "ifrh_mex_run requires an options argument." );
    }

    /* Is input not complex? */
    if( !mxIsComplex( mp_rhs[0] ) )
    {
        /* Exit with error message */
        mexErrMsgTxt( "Input should be complex." );
    }

    /* Read options structure. Failed? */
    if( IFRH_ImportOptions( mp_rhs[1], sp_opt ) == IFRH_FAILURE )
    {
        /* Exit with error message */
        mexPrintf( "%s\n", MXR_GetError( ) );
        mexErrMsgTxt( "Failed to read options structure (see above)." );
    }

    /* Get pointers to input */
    mp_in    = mp_rhs[0];
    dp_in_re = mxGetPr( mp_in );
    dp_in_im = mxGetPi( mp_in );

    /* Get number of rows and columns in input */
    n_in_row = mxGetM( mp_in );
    n_in_col = mxGetN( mp_in );

    /* Inconsistent number of rows in input and channels option? */
    if( n_in_row != sp_opt->n_channels )
    {
        /* Exit with error message */
        mexErrMsgTxt( "Number of input channels does not match options." );
    }

    /* Allocate memory for state. Failed? */
    if( IFRH_AllocState( sp_state, sp_opt ) == IFRH_FAILURE )
    {
        /* Exit with error message */
        mexErrMsgTxt( "Failed to allocate memory for state." );
    }

    /* Third argument supplied? */
    if( n_rhs >= 3 )
    {
        /* Import state */
        int n_result = IFRH_ImportState( mp_rhs[2], sp_opt, sp_state );

        /* Failed? */
        if( n_result == IFRH_FAILURE )
        {
            /* Free the allocated memory */
            IFRH_FreeState( sp_state );

            /* Exit with error message */
            mexPrintf( "%s\n", MXR_GetError( ) );
            mexErrMsgTxt( "Failed to read state structure." );
        }
    }
    else
    {
        /* Initialise state */
        IFRH_InitState( sp_state, sp_opt );
    }

    /* Get number of output rows and columns */
    n_out_row = sp_opt->n_bins;
    n_out_col = IFRH_CalcOutputSize( n_in_col, sp_state, sp_opt );

    /* Make a real double matrix to store histogram */
    mp_out = mxCreateDoubleMatrix( n_out_row, n_out_col, mxREAL );
    dp_out = mxGetPr( mp_out );

    /* Run IFRH */
    IFRH_Run( dp_in_re, dp_in_im, dp_out, n_in_col, sp_state, sp_opt );

    /* Request outputs? */
    if( n_lhs > 0 ) mp_lhs[0] = mp_out;
    if( n_lhs > 1 ) mp_lhs[1] = IFRH_ExportState( sp_state, sp_opt );

    /* Free state memory */
    IFRH_FreeState( sp_state );
}

/* end of file */
