/* mx_read.c */

/* Support for data conversions from Matlab to C.
 * Author: Robert Mill <rob.mill.uk@gmail.com>
 * (C)2015 University of Sheffield
 */

/* INCLUDER HEADERS */

#include <stdio.h>
#include <string.h>
#include "mx_read.h"


/* CONSTANTS */

enum {N_ERR_MSG_LEN = 255};


/* GLOBAL VARIABLES */

static char cp_error_msg[N_ERR_MSG_LEN + 1];


/* FUNCTION DEFINITIONS */

int MXR_IsError( void )
{
    return cp_error_msg[0] != (char) 0;
}


void MXR_ClearError( void )
{
    cp_error_msg[0] = '\0';
    cp_error_msg[N_ERR_MSG_LEN] = '\0';
}


void MXR_SetError( const char *cp_msg )
{
    strncpy( cp_error_msg, cp_msg, N_ERR_MSG_LEN );
}


char * MXR_GetError( void )
{
    return cp_error_msg;
}


const mxArray * MXR_CheckType(
    const mxArray *mp_array,
    const char *cp_name,
    const char *cp_class )
{
    if( mp_array && !mxIsClass( mp_array, cp_class ) )
    {
        sprintf(
            cp_error_msg,
            "\"%s\" should be of type %s.",
            cp_name,
            cp_class );

        mp_array = NULL;
    }

    return mp_array;
}


const mxArray * MXR_CheckDims(
    const mxArray *mp_array,
    const char *cp_name,
    int n_row, 
    int n_col )
{
    if( mp_array )
    {
        int n_M = mxGetM( mp_array );
        int n_N = mxGetN( mp_array );

        if( n_row != n_M || n_col != n_N )
        {
            sprintf(
                cp_error_msg,
                "\"%s\" should be of size %d x %d.",
                cp_name,
                n_row,
                n_col );

            mp_array = NULL;
        }
    }

    return mp_array;
}


const mxArray * MXR_CheckComplex( const mxArray *mp_array, const char *cp_name )
{
    if( mp_array && !mxIsComplex( mp_array ) )
    {
        sprintf( cp_error_msg, "\"%s\" should be complex.", cp_name );
        mp_array = NULL;
    }

    return mp_array;
}


const mxArray * MXR_GetField(
    const mxArray *mp_struct,
    const char *cp_field,
    int n_index )
{
    const mxArray * mp_field = cp_field[0] == '<' ?
        mp_struct :
        mxGetField( mp_struct, n_index, cp_field );

    if( mp_field == NULL )
    {
        sprintf(
            cp_error_msg,
            "Missing field: %s.",
            cp_field );
    }

    return mp_field;
}


const int * MXR_GetInt32Ptr(
    const mxArray *mp_struct,
    const char *cp_field,
    int n_index )
{
    const mxArray * mp_field;

    mp_field = MXR_GetField( mp_struct, cp_field, n_index );
    mp_field = MXR_CheckType( mp_field, cp_field, "int32" );

    return mp_field ? (int *) mxGetData( mp_field ) : NULL;
}


const double * MXR_GetDoublePtr(
    const mxArray *mp_struct,
    const char *cp_field, 
    int n_index )
{
    const mxArray * mp_field;

    mp_field = MXR_GetField( mp_struct, cp_field, n_index );
    mp_field = MXR_CheckType( mp_field, cp_field, "double" );

    return mp_field ? (double *) mxGetData( mp_field ) : NULL;
}


const double * MXR_GetRealPtr(
    const mxArray *mp_struct,
    const char *cp_field, 
    int n_index )
{
    const mxArray * mp_field;

    mp_field = MXR_GetField( mp_struct, cp_field, n_index );
    mp_field = MXR_CheckType( mp_field, cp_field, "double" );
    mp_field = MXR_CheckComplex( mp_field, cp_field );

    return mp_field ? (double *) mxGetPr( mp_field ) : NULL;
}


const double * MXR_GetImagPtr(
    const mxArray *mp_struct,
    const char *cp_field,
    int n_index )
{
    const mxArray * mp_field;

    mp_field = MXR_GetField( mp_struct, cp_field, n_index );
    mp_field = MXR_CheckType( mp_field, cp_field, "double" );
    mp_field = MXR_CheckComplex( mp_field, cp_field );

    return mp_field ? (double *) mxGetPi( mp_field ) : NULL;
}


int MXR_GetNumEl(
    const mxArray *mp_struct,
    const char *cp_field,
    int n_index )
{
    const mxArray * mp_field = MXR_GetField( mp_struct, cp_field, n_index );

    return mp_field ? mxGetNumberOfElements( mp_field ) : 0;
}


void MXR_GetInt32(
    const mxArray *mp_struct,
    const char *cp_field, 
    int n_index,
    int n_numel,
    int *np_dest )
{
    const int *np_src = MXR_GetInt32Ptr( mp_struct, cp_field, n_index );

    if( np_src )
    {
        memcpy( np_dest, np_src, n_numel * sizeof(int) );
    }
}


void MXR_GetDouble(
    const mxArray *mp_struct,
    const char *cp_field, 
    int n_index,
    int n_numel, 
    double *dp_dest )
{
    const double *dp_src = MXR_GetDoublePtr( mp_struct, cp_field, n_index );

    if( dp_src )
    {
        memcpy( dp_dest, dp_src, n_numel * sizeof(double) );
    }
}


void MXR_GetComplex(
    const mxArray *mp_struct,
    const char *cp_field, 
    int n_index,
    int n_numel,
    double *dp_dest_re, 
    double *dp_dest_im )
{
    const double *dp_src_re = MXR_GetRealPtr( mp_struct, cp_field, n_index );
    const double *dp_src_im = MXR_GetImagPtr( mp_struct, cp_field, n_index );

    if( dp_src_re && dp_src_im )
    {
        memcpy( dp_dest_re, dp_src_re, n_numel * sizeof(double) );
        memcpy( dp_dest_im, dp_src_im, n_numel * sizeof(double) );
    }
}

/* end of file */
