/* mx_swdft.c */

/* MEX interface support for the SWDFT.
 * Author: Robert Mill <rob.mill.uk@gmail.com>
 * (C)2015 University of Sheffield
 */

/* INCLUDE HEADERS */

#include <mex.h>
#include "mx_read.h"
#include "mx_write.h"
#include "mx_swdft.h"


/* FUNCTION DEFINITIONS */

int SWD_ImportOptions( const mxArray *mp_opt, SWD_Options *sp_opt )
{
    /* Clear error messages */
    MXR_ClearError( );

    /* Are options not provided in a structure? */
    if( !mxIsStruct( mp_opt ) )
    {
        /* Set error message and return failure */
        MXR_SetError( "Options should be a structure." );
        return SWD_FAILURE;
    }

    /* Read SWD options from structure */
    MXR_GetInt32( mp_opt, "n_dft_len", 0, 1, &sp_opt->n_dft_len );
    MXR_GetInt32( mp_opt, "n_cell_start", 0, 1, &sp_opt->n_cell_start );
    MXR_GetInt32( mp_opt, "n_cell_count", 0, 1, &sp_opt->n_cell_count );
    MXR_GetInt32( mp_opt, "n_fringe", 0, 1, &sp_opt->n_fringe );
    MXR_GetDouble( mp_opt, "d_leak", 0, 1, &sp_opt->d_leak );

    /* Read filter options from structure */
    sp_opt->n_fwin_len = MXR_GetNumEl( mp_opt, "dc_f_win", 0 );
    sp_opt->dp_fwin_re = MXR_GetRealPtr( mp_opt, "dc_f_win", 0 );
    sp_opt->dp_fwin_im = MXR_GetImagPtr( mp_opt, "dc_f_win", 0 );

    /* Return status */
    return MXR_IsError( ) ? SWD_FAILURE : SWD_SUCCESS;
}


int SWD_ImportState( 
    const mxArray *mp_state, 
    const SWD_Options *sp_opt,
    SWD_State *sp_state )
{
    int n_dft   = sp_opt->n_dft_len;
    int n_cells = sp_opt->n_cell_count + 2*sp_opt->n_fringe;

    /* Clear error messages */
    MXR_ClearError( );

    /* Is state not provided in a structure? */
    if( !mxIsStruct( mp_state ) )
    {
        /* Set error message and return failure */
        MXR_SetError( "State should be a structure." );
        return SWD_FAILURE;
    }

    /* Read buffer R/W index */
    MXR_GetInt32( mp_state, "n_buf_pos", 0, 1, &sp_state->n_buf_pos );

    /* Read buffer state */
    MXR_GetDouble( mp_state, "dp_buffer", 0, n_dft, sp_state->dp_buffer );

    /* Read DFT cell states */
    MXR_GetComplex(
        mp_state,
        "dp_cells",
        0,
        n_cells,
        sp_state->dp_cells_re,
        sp_state->dp_cells_im );

    /* Read filter states */
    MXR_GetComplex(
        mp_state,
        "dp_filt",
        0,
        n_cells,
        sp_state->dp_filt_re,
        sp_state->dp_filt_im );

    /* Return status */
    return MXR_IsError( ) ? SWD_FAILURE : SWD_SUCCESS;
}


mxArray * SWD_ExportState(
    const SWD_State *sp_state,
    const SWD_Options *sp_opt )
{
    int n_cells = sp_opt->n_cell_count + 2*sp_opt->n_fringe;

    /* Create a state structure to add fields to */
    mxArray *mp_state = mxCreateStructMatrix( 1, 1, 0, NULL );

    /* Clear error messages */
    MXW_ClearError( );

    /* Write buffer R/W index */
    MXW_PutInt32( mp_state, "n_buf_pos", &sp_state->n_buf_pos, 1, 1 );

    /* Write buffer state */
    MXW_PutDouble(
        mp_state,
        "dp_buffer",
        sp_state->dp_buffer,
        sp_opt->n_dft_len,
        1 );

    /* Write DFT cell states */
    MXW_PutComplex(
        mp_state,
        "dp_cells",
        sp_state->dp_cells_re,
        sp_state->dp_cells_im,
        n_cells,
        1 );

    /* Write filter states */
    MXW_PutComplex(
        mp_state,
        "dp_filt",
        sp_state->dp_filt_re,
        sp_state->dp_filt_im,
        n_cells,
        1 );

    /* Return status */
    return MXW_IsError( ) ? NULL : mp_state;
}

/* end of file */
