/* mx_swdft_run.c */

/* Code invocable from Matlab to produce sliding window discrete Fourier
 * transform.
 * Author: Robert Mill <rob.mill.uk@gmail.com>
 * (C)2015 University of Sheffield
 */

/* INCLUDE HEADERS */

#include <mex.h>
#include "mx_read.h"
#include "mx_swdft.h"


/* FUNCTION DEFINITIONS */

void mexFunction(
    int n_lhs,
    mxArray *mp_lhs[],
    int n_rhs,
    const mxArray *mp_rhs[] )
{
    int n_in;
    int n_cells;

    /* Inputs */
    const mxArray *mp_in;
    double        *dp_in;

    /* Outputs */
    mxArray *mp_out;
    double  *dp_out_re;
    double  *dp_out_im;

    /* Memory for options and state */
    SWD_Options sp_opt   [1];
    SWD_State   sp_state [1];

    /* Not enough arguments? */
    if( n_rhs < 2 )
    {
        /* Exit with error message */
        mexErrMsgTxt( "swdft_mex_run requires an options structure." );
    }

    /* Read options structure. Failed? */
    if( SWD_ImportOptions( mp_rhs[1], sp_opt ) == SWD_FAILURE )
    {
        /* Exit with error message */
        mexPrintf( "%s\n", MXR_GetError( ) );
        mexErrMsgTxt( "Failed to read options structure." );
    }

    /* Get number of DFT cells from options */
    n_cells = sp_opt->n_cell_count;

    /* Allocate memory for state. Failed? */
    if( SWD_AllocState( sp_state, sp_opt ) == SWD_FAILURE )
    {
        /* Exit with error message */
        mexErrMsgTxt( "Failed to allocate memory for state." );
    }

    /* Third argument supplied? */
    if( n_rhs >= 3 )
    {
        /* Import state. Failed? */
        if( SWD_ImportState( mp_rhs[2], sp_opt, sp_state ) == SWD_FAILURE )
        {
            /* Free the allocated memory */
            SWD_FreeState( sp_state );

            /* Exit with error message */
            mexPrintf( "%s\n", MXR_GetError( ) );
            mexErrMsgTxt( "Failed to read state structure." );
        }
    }
    else
    {
        /* Initialise */
        SWD_InitState( sp_state, sp_opt );
    }

    /* Get pointers to input and determine input size */
    mp_in = mp_rhs[0];
    dp_in = mxGetPr( mp_in );
    n_in  = mxGetNumberOfElements( mp_in );

    /* Make a complex double matrix to store DFT */
    mp_out    = mxCreateDoubleMatrix( n_cells, n_in, mxCOMPLEX );
    dp_out_re = mxGetPr( mp_out );
    dp_out_im = mxGetPi( mp_out );

    /* Run SWD */
    SWD_Run( dp_in, dp_out_re, dp_out_im, n_in, sp_state, sp_opt );

    /* Request outputs? */
    if( n_lhs > 0 ) mp_lhs[0] = mp_out;
    if( n_lhs > 1 ) mp_lhs[1] = SWD_ExportState( sp_state, sp_opt );

    /* Free state memory */
    SWD_FreeState( sp_state );
}

/* end of file */
