/* mx_zcpa.c */

/* MEX interface support for the ZCPA.
 * Author: Robert Mill <rob.mill.uk@gmail.com>
 * (C)2015 University of Sheffield
 */

/* INCLUDE HEADERS */

#include <mex.h>
#include <stdlib.h>
#include "mx_read.h"
#include "mx_write.h"
#include "mx_zcpa.h"


/* FUNCTION DEFINITIONS */

void ZCPA_ImportMaps( const mxArray *mp_opt, ZCPA_Options *sp_opt )
{
    int j;
    int n_channels = sp_opt->n_channels;

    /* Set pointer to maps to NULL */
    sp_opt->sp_maps = NULL;

    /* Return if assignment mode does not use remapping */
    if( sp_opt->n_ass_mode != ZCPA_ASS_REMAP ) return;

    /* Check number of maps matches number of bins */
    MXR_CheckDims(
        MXR_GetField( mp_opt, "sp_maps", 0 ),
        "sp_maps",
        sp_opt->n_channels,
        1 );

    /* Return if error */
    if( MXR_IsError( ) ) return;

    /* Allocate space for maps */
    sp_opt->sp_maps = calloc( n_channels, sizeof(ZCPA_Map) );

    /* For each channel/map j: */
    for( j = 0; j < n_channels; j++ )
    {
        /* Get pointer to map */
        const mxArray *mp_maps = MXR_GetField( mp_opt, "sp_maps", 0 );
        ZCPA_Map *sp_map  = sp_opt->sp_maps + j;

        /* Get map fields */
        MXR_GetInt32( mp_maps, "n_bin_start", j, 1, &sp_map->n_bin_start );
        MXR_GetInt32( mp_maps, "n_bin_count", j, 1, &sp_map->n_bin_count );
        MXR_GetInt32(
            mp_maps,
            "n_zci_count",
            j,
            1,
            &sp_map->n_zci_count );

        /* ... get map fields ... */
        MXR_GetDouble(
            mp_maps,
            "d_zci_start",
            j,
            1,
            &sp_map->d_zci_start );

        /* ... get map fields ... */
        MXR_GetDouble(
            mp_maps,
            "d_zci_width",
            j,
            1,
            &sp_map->d_zci_width );

        /* No errors up to this point? */
        if( !MXR_IsError( ) )
        {
            /* Check the dimensions of the PDF */
            MXR_CheckDims(
                MXR_GetField( mp_maps, "dp_pdf", j ),
                "dp_pdf",
                sp_map->n_bin_count, 
                sp_map->n_zci_count );

            /* Get pointer to PDF */
            sp_map->dp_pdf = MXR_GetDoublePtr( mp_maps, "dp_pdf", j );
        }
    }

    /* Any error occurred to up to this point? */
    if( MXR_IsError( ) )
    {
        /* Free space reserved for maps */
        free( sp_opt->sp_maps );
        sp_opt->sp_maps = 0;
    }
}


void ZCPA_FreeImportedMaps( ZCPA_Options *sp_opt )
{
    /* Options are not null? */
    if( sp_opt != NULL )
    {
        /* Free map data */
        if( sp_opt->sp_maps ) free( sp_opt->sp_maps );
        sp_opt->sp_maps = NULL;
    }
}


int ZCPA_ImportOptions( const mxArray *mp_opt, ZCPA_Options *sp_opt )
{
    /* Clear error messages */
    MXR_ClearError( );

    /* Are options not provided in a structure? */
    if( !mxIsStruct( mp_opt ) )
    {
        /* Set error message and return failure */
        MXR_SetError( "Options should be a structure." );
        return ZCPA_FAILURE;
    }

    /* Read ZCPA options from structure (ints) */
    MXR_GetInt32( mp_opt, "n_fs_Hz", 0, 1, &sp_opt->n_fs_Hz );
    MXR_GetInt32( mp_opt, "n_channels", 0, 1, &sp_opt->n_channels );
    MXR_GetInt32( mp_opt, "n_bins", 0, 1, &sp_opt->n_bins );
    MXR_GetInt32( mp_opt, "n_ass_mode", 0, 1, &sp_opt->n_ass_mode );
    MXR_GetInt32( mp_opt, "n_peak_mode", 0, 1, &sp_opt->n_peak_mode );
    MXR_GetInt32( mp_opt, "n_out_wait", 0, 1, &sp_opt->n_out_wait );

    /* Read ZCPA options from structure (doubles) */
    MXR_GetDouble( mp_opt, "d_freq_start", 0, 1, &sp_opt->d_freq_start );
    MXR_GetDouble( mp_opt, "d_freq_width", 0, 1, &sp_opt->d_freq_width );

    /* Read bin decay time constants */
    sp_opt->dp_taus_s = MXR_GetDoublePtr( mp_opt, "dp_taus_s", 0 );

    /* Check number of time constants matches number of bins */
    MXR_CheckDims(
        MXR_GetField( mp_opt, "dp_taus_s", 0 ),
        "dp_taus_s",
        sp_opt->n_bins,
        1 );

    /* No error occurred ? */
    if( !MXR_IsError( ) )
    {
        /* Import maps */
        ZCPA_ImportMaps( mp_opt, sp_opt );
    }

    /* Return status */    
    return MXR_IsError( ) ? ZCPA_FAILURE : ZCPA_SUCCESS;
}


int ZCPA_ImportState( 
    const mxArray *mp_state, 
    const ZCPA_Options *sp_opt,
    ZCPA_State *sp_state )
{
    int n_channels = sp_opt->n_channels;
    int n_bins     = sp_opt->n_bins;

    /* Clear error messages */
    MXR_ClearError( );

    /* Is state not provided in a structure? */
    if( !mxIsStruct( mp_state ) )
    {
        /* Set error message and return failure */
        MXR_SetError( "State should be a structure." );
        return ZCPA_FAILURE;
    }

    /* Read state */
    MXR_GetInt32( mp_state, "n_out_rem", 0, 1, &sp_state->n_out_rem );
    MXR_GetInt32( mp_state, "np_zci", 0, n_channels, sp_state->np_zci );
    MXR_GetDouble( mp_state, "dp_in_z", 0, n_channels, sp_state->dp_in_z );
    MXR_GetDouble( mp_state, "dp_peak", 0, n_channels, sp_state->dp_peak );
    MXR_GetDouble(
        mp_state,
        "dp_zci_frac",
        0,
        n_channels,
        sp_state->dp_zci_frac );
    MXR_GetDouble( mp_state, "dp_hist", 0, n_bins, sp_state->dp_hist );

    /* Return status */
    return MXR_IsError( ) ? ZCPA_FAILURE : ZCPA_SUCCESS;
}


mxArray * ZCPA_ExportState(
    const ZCPA_State *sp_state,
    const ZCPA_Options *sp_opt )
{
    int n_channels = sp_opt->n_channels;
    int n_bins     = sp_opt->n_bins;

    /* Create a state structure to add fields to */
    mxArray *mp_state = mxCreateStructMatrix( 1, 1, 0, NULL );

    /* Clear error messages */
    MXW_ClearError( );

    /* Read state */
    MXW_PutInt32( mp_state, "n_out_rem", &sp_state->n_out_rem, 1, 1 );
    MXW_PutInt32( mp_state, "np_zci", sp_state->np_zci, n_channels, 1 );
    MXW_PutDouble( mp_state, "dp_in_z", sp_state->dp_in_z, n_channels, 1 );
    MXW_PutDouble( mp_state, "dp_peak", sp_state->dp_peak, n_channels, 1 );
    MXW_PutDouble(
        mp_state,
        "dp_zci_frac",
        sp_state->dp_zci_frac,
        n_channels,
        1 );
    MXW_PutDouble( mp_state, "dp_hist", sp_state->dp_hist, n_bins, 1 );

    /* Return status */
    return MXW_IsError( ) ? NULL : mp_state;
}

/* end of file */
